"use strict";

//var config = require('./config');
var crypto = require('crypto');
var fs = require('fs');
var SunCalc = require('suncalc');

var baseDevice = require('./devices/device');

var devices = {};
var lastModified;
var stick;
var settings;
var writeWait;

var funcGetLastModified = function() {
    return lastModified;
}

var funcGetFullDevices = function() {
    var result = [];
    for(var idx in devices) {
        result.push(devices[idx]);
    }
    return result;
}

var funcGetDevice = function(id) {
    return devices[id];
}

var funcSetDevice = function(id, device) {
    if(id == undefined) {
        id = generateId();
    }
    devices[id] = device;
    save();
}

var funcDeleteDevice = function(id) {
    if(id.startsWith("CGW.")) {
        stick.delete(id);
        
        var result = [];
        for(var idx in devices) {
            var device = devices[idx];
            if(device.type == "scene") {
                for(midx in device.members) {
                    if(device.members[midx].id == id) {
                        device.members.splice(midx, 1);
                    }
                }
            }
            
            if(device.type == "group") {
                for(var midx in device.members) {
                    if(device.members[midx] == id) {
                        device.members.splice(midx, 1);
                    }
                }
            }
        }
    }

    delete devices[id];
    save();
}

var funcSetDeviceState = function(id, state) {
    setDeviceStateWithMode(id, state, false);
}

function setDeviceStateWithMode(id, state, mode) {
    if(id == "CGW" && state == "pair" && devices[id].state != "na") {
        stick.startScan();
    }
    
    devices[id].state = state;

    if(id.startsWith("CGW.")) {
        var s = state.split(" ");
        if(s.length == 2) {
            stick.runCommand(id, s[0], s[1], mode);
        } else {
            stick.runCommand(id, s[0], 0, mode);
        }
    }

    if(devices[id].type == "group") {
        //TODO update member states
        var grp = devices[id];

        var s = state.split(" ");
        if(s.length == 2) {
            stick.runGroupCommand(grp.members, s[0], s[1], mode);
        } else {
            stick.runGroupCommand(grp.members, s[0], 0, mode);
        }
    }

    if(devices[id].type == "scene") {
        //TODO update member states
        var scene = devices[id];

        if(state == "stop") {
            var memberIds = [];
            for(var idx in scene.members) {
                var dev = scene.members[idx];
                memberIds.push(dev.id);
            }
            stick.runGroupCommand(memberIds, "stop", 0, mode);
        } else {
            var cmds = {};
            var seqCmds = [];
            for(var idx in scene.members) {
                var dev = scene.members[idx];
                var s = dev.value.split(" ");
                if(s.length == 2) {
                    var scmd = {};
                    scmd.id = dev.id;
                    scmd.p1 = s[0];
                    scmd.p2 = s[1];
                    
                    seqCmds.push(scmd);
                    //stick.runCommand(dev.id, s[0], s[1], mode);
                } else {
                    if(cmds[s[0]] == undefined) {
                        cmds[s[0]] = [];
                    }
                    
                    cmds[s[0]].push(dev.id);
                    //stick.runCommand(dev.id, s[0], 0, mode);
                }
            }
            console.log("CMDs: " + JSON.stringify(cmds));
            console.log("SeqCMDs: " + JSON.stringify(seqCmds));
            
            for(var idx in cmds) {
                stick.runGroupCommand(cmds[idx], idx, 0, mode);
            }
            
            for(var idx in seqCmds) {                
                stick.runCommand(seqCmds[idx].id, seqCmds[idx].p1, seqCmds[idx].p2, mode);
            }
        }
    }

    /*if(devices[id].type == "timer") {
        checkTimer();
        updateTimer();
    }*/
}


var funcCreateDevice = function(device) {
    var err = baseDevice.check(device);

    if(err == undefined) {
        if(device.id == undefined) {
            device.id = generateId();
        }


        var dev = baseDevice.create(device);

        if(devices[dev.id] != undefined) {
            if(dev.id.startsWith("CGW.") && devices[dev.id].name != dev.name) {
                stick.updateName(dev.id, dev.name);
            }
        }

        devices[dev.id] = dev;
        save();
    }

    return device.id;
}

var funcCheckDevice = function(device) {
	return baseDevice.check(device);
}

function saveInstant() {
    lastModified = new Date();
    fs.renameSync('/opt/chs/devices.json', '/opt/chs/devices.json.old');
    fs.writeFile('/opt/chs/devices.json', JSON.stringify(devices), function(err) {
        if(err) {
            console.log(err);
        } else {
            console.log("The file was saved!");
        }
    });
}

function save() {
    lastModified = new Date();
    if(writeWait == undefined) {
        writeWait = setTimeout(function() {
    	    fs.renameSync('/opt/chs/devices.json', '/opt/chs/devices.json.old');
    	    fs.writeFile('/opt/chs/devices.json', JSON.stringify(devices), function(err) {
                if(err) {
               	    console.log(err);
                } else {
           	     console.log("The file was saved!");
                }
    	    });
	    writeWait = undefined;
	}, 60000);
    }
}

function generateId() {
    return crypto.randomBytes(20).toString('hex');
}

function timeRunner() {
    setTimeout(function() {
        checkTimer();
        updateTimer();
        timeRunner();
    }, 10000);
}

function getTimer() {
    var result = [];
    for(var idx in devices) {
        if(devices[idx].type == "timer") {
            result.push(devices[idx]);
        }
    }

    return result;
}

function checkTimer() {
    var timers = getTimer();
    for(var idx in timers) {
        var timer = timers[idx];
        if(timer.device != undefined && devices[timer.device] == undefined) {
            delete devices[timer.id];
            save();
        } else {
            if(timer.state == "on") {
                if(timer.readings != undefined && timer.readings.next != undefined) {
                    if(typeof(timer.readings.next) == "string") {
                        timer.readings.next = new Date(timer.readings.next);
                    }

                    if(timer.readings.next < new Date()) {
                        timer.readings.next = undefined;
                        setDeviceStateWithMode(timer.device, timer.device_state, true);
                    }
                }
            } else {
                if(timer.readings != undefined) {
                    timer.readings.next = undefined;
                }
            }
        }
    }
}

function updateTimer() {
    var timers = getTimer();
    for(var idx in timers) {
        var timer = timers[idx];
        if(timer.readings == undefined) {
            timer.readings = {};
        }

        if(timer.state == "on" && timer.readings.next == undefined) {
            timer.readings.next = calculateTimer(timer);
            console.log("next " + timer.readings.next)
        }
    }
}

function calculateStaticTimer(date, timer) {
    if(timer.time == undefined) return undefined;

    var timeSplit = timer.time.split(":");

    if(timeSplit.length == 2) {
        date.setMinutes(timer.time.split(":")[1]);
        date.setHours(timer.time.split(":")[0]);

        return date;
    }

    return undefined;

}

function calculateRandomTimer(date, timer) {
    if(timer.time == undefined) return undefined;

    var timeSplit = timer.time.split(":");

    if(timeSplit.length == 2) {
        date.setMinutes(timer.time.split(":")[1]);
        date.setHours(timer.time.split(":")[0]);

        var offset = Math.floor((Math.random() * 31));
        offset = offset - 15;

        date.setMinutes(date.getMinutes()+offset);

        return date;
    }

    return undefined;
}

function calculateAstroTimer(date, timer, sunrise) {
    var location = settings.getSettings().location;
    if(location == undefined || location.latitude == undefined || location.longitude == undefined) {
        location = {};
        location.latitude = 50.9;
        location.longitude = 7.0;
    }

    var times = SunCalc.getTimes(date, location.latitude, location.longitude);

    if(sunrise) {
        date = times.sunrise;
    } else {
        date = times.sunset;
    }

    if(timer.offset != undefined && timer.offset != null) {
        date.setMinutes(date.getMinutes()+parseInt(timer.offset));
    }

    if(timer.time != undefined && timer.time != null) {
        var t = timer.time.split(":");
        if(sunrise) {
            if(t[0] > date.getHours() || (t[0] == date.getHours() && t[1] > date.getMinutes())) {
                date.setHours(t[0]);
                date.setMinutes(t[1]);
            }
        } else {
            if(t[0] < date.getHours() || (t[0] == date.getHours() && t[1] < date.getMinutes())) {
                date.setHours(t[0]);
                date.setMinutes(t[1]);
            }
        }
    }

    console.log("date: " + date);

    return date;
}

function calculateNextDate(timer, includeToday) {
    var date = new Date();
    date.setHours(12);
    date.setMinutes(0);
    date.setSeconds(0);
    date.setMilliseconds(0);

    if(includeToday == false) {
        date.setDate(date.getDate()+1);
    }

    var day = date.getDay();

    var dayOffset = 0;

    while(timer.days[(day+dayOffset)%7] != "1" && dayOffset < 8) {
        dayOffset++;
    }

    if(timer.days[(day+dayOffset)%7] == "1") {
        date.setDate(date.getDate()+dayOffset);
        return date;
    } else {
        return undefined;
    }

}

function calculateTimer(timer) {
    var result;
    var date = calculateNextDate(timer, true);
    if(date == undefined || date.getFullYear() < 2016) {
        return undefined;
    }

    result = getTimeForDate(date, timer);

    if(result == undefined || result < new Date()) {
        date = calculateNextDate(timer, false);
        result = getTimeForDate(date, timer);
    }

    if(result != undefined) {
        lastModified = new Date();
    }

    return result;

}

function getTimeForDate(date, timer) {
    if(timer.timer_type == undefined || timer.timer_type == "static") {
        return calculateStaticTimer(date, timer);
    } else if(timer.timer_type == "random") {
        return calculateRandomTimer(date, timer);
    } else if(timer.timer_type == "astroSunset") {
        return calculateAstroTimer(date, timer, false);
    } else if(timer.timer_type == "astroSunrise") {
        return calculateAstroTimer(date, timer, true);
    }

    return undefined;
}

function checkMySelveStatus() {
    var myselConf = settings.getSettings().myselve;
    if(myselConf.password == undefined) {
        var https = require('https');
        var options = {
            hostname: 'www.my-selve.de',
            port: 443,
            path: '/api',
            method: 'POST',
            headers: {'Content-Type': 'application/json'}
        };
        
        var config = {}
        config.code = myselConf.activationcode;
        config.name = myselConf.name;
        config.mac = settings.getSettings().mac.replace(/:/g, '');
        config.description = myselConf.descr;
        
        var req = https.request(options, function(res) {
            res.on('data', function(d) {
                    console.log('statusCode:', res.statusCode);
                    console.log('headers:', res.headers);
            
                    if(res.statusCode != 200) {
                        settings.del("myselve");
                    } else {
                        var r = d.toString('utf8');
                        console.log(r);
                        handleMySelveResponse(r);
                    }
            }).on('error', function (err) {
                settings.del("myselve");
            });
        });
        
        req.end(JSON.stringify(config));
    }
}

function handleMySelveResponse(response) {
    var res = JSON.parse(response);
    var myselConf = settings.getSettings().myselve;
    
    myselConf.password = res.password;
    myselConf.domain = res.domain;
    myselConf.deviceid = res.deviceid;
    
    settings.put("myselve", myselConf);
    writeTunnelSettings(myselConf);
    startTunnel();
}

function unregisterMySelve(data) {
    var https = require('https');
    var options = {
        hostname: 'www.my-selve.de',
        port: 443,
        path: '/api',
        method: 'POST',
        headers: {'Content-Type': 'application/json'}
    };
        
    var config = {}
    config.password = data.password;
    config.deviceid = data.deviceid;
        
    var req = https.request(options, function(res) {
        console.log('statusCode:', res.statusCode);
        console.log('headers:', res.headers);
            
        res.on('data', function(d) {
            var r = d.toString('utf8');
            console.log(r);
            writeTunnelSettings()
        });
    });
        
    req.end(JSON.stringify(config));
}


function writeTunnelSettings(conf) {
    var data = "";
    if(conf != undefined) {
        data += "webtunnel.domain = " + conf.domain + "\n";
        data += "webtunnel.deviceId = " + conf.deviceid + "\n";
        data += "webtunnel.reflectorURI = https://reflector.my-selve.de\n";
        data += "webtunnel.username = ${webtunnel.deviceId}@${webtunnel.domain}\n";
        data += "webtunnel.password = " + conf.password + "\n";
        data += "webtunnel.ports = 80;22\n"; 
    } 
    fs.writeFileSync('/opt/chs/iveotunnelserver.properties', data);
}

function startTunnel() {
    var process = require('child_process');
    var child= process.exec("systemctl restart chstunnel");
}

var funcSettingsChange = function(key, data) {
    console.log("SettingsChange with " + key + " " + data);
    if(key == "myselve" && data == undefined) {
        checkMySelveStatus();
    } else if(key == "myselve" && data != undefined) {
        unregisterMySelve(data);
    }
    
    if(key == "location" && data == undefined) {
        resetAstroTimer();
    }
}

function resetAstroTimer() {
    var timers = getTimer();
    for(var idx in timers) {
        var timer = timers[idx];
        if(timer.timer_type == "astroSunset" || timer.timer_type == "astroSunrise") {
            timer.readings = {};
        }
    }
}

var funcImportBackup = function(d) {
    devices = {};
    for(var idx in d) {
        var dev = d[idx];
        devices[dev.id] = dev;
    }
    saveInstant();
}

var exp =
{
        getDevice: funcGetDevice,
        checkDevice: funcCheckDevice,
        createDevice: funcCreateDevice,
        deleteDevice: funcDeleteDevice,
        setDeviceState: funcSetDeviceState,
        getFullDevices: funcGetFullDevices,
        lastModified: funcGetLastModified,
        importBackup: funcImportBackup
}

function readOldFile() {
  console.log("try to read old devices");
  var fd = fs.readFileSync('/opt/chs/devices.json.old');
  try {
    return JSON.parse(fd);
  } catch(err) {
    console.log(err);
  }
  return {};
}

module.exports = function(settingsData) {
    fs.readFile('/opt/chs/devices.json', 'utf8', function (err,file) {
        if (err) {
            console.log(err)
            devices = readOldFile();
        } else {
            try {
                devices = JSON.parse(file);
            } catch(err) {
      		console.log(err)
                devices = readOldFile();
            }
        }

        stick = require("./stick/stick_new.js")(exp);
    });

    settings = settingsData;
    settings.onChange(funcSettingsChange);

    timeRunner();

    return exp;
}
