"use strict";

var chs;

var types = ["shutter", "jalousie", "marquee", "dimmer", "night-switch", "twilight-switch", "heating", "cooling", "day-switch", "timer", "scene", "group", "room", "gateway", "real"];
var realTypes = ["shutter", "jalousie", "marquee", "dimmer", "night-switch", "twilight-switch", "heating", "cooling", "day-switch"];

var funcList = function(req, res, next) {  
    var etag = req.header('If-None-Match', '0');
    
    if(etag == chs.lastModified().getTime()) {
        res.send(304);
        return;
    }
    
    res.header('ETag', chs.lastModified().getTime());
    var devices = chs.getFullDevices()
    
    if(req.params.filter != undefined) {
        if(types.indexOf(req.params.filter) == -1) {
            res.send(400, "filter type unknown. (" + types + ")");
            return;
        }
        
        var fResult = [];
        for(var idx in devices) {
            if(devices[idx].type == req.params.filter || (req.params.filter == "real" && realTypes.indexOf(devices[idx].type) != -1)) {
                fResult.push(devices[idx]);
            }
        }
        
        res.send(200, fResult);
    } else {
        res.send(200, devices);
    }
}

var funcView = function(req, res, next) {
    var dev = chs.getDevice(req.params.id);
    if(dev != undefined) {
        res.send(200, dev);
    } else {
        res.send(400, dev);
    }
}

var funcAdd = function(req, res, next) {
    var device = {};
    device.id = req.params.id;
    device.type = req.params.type;
    device.name = req.params.name;
    device.room = req.params.room;
    device.state = req.params.state;
    device.time = req.params.time;
    device.sort = req.params.sort;
    device.sort_dev = req.params.sort_dev;
    device.timer_type = req.params.timer_type;
    device.offset = req.params.offset;
    device.days = req.params.days;
    device.members = req.params.members;
    device.device = req.params.device;
    device.device_state = req.params.device_state;
    
    var prevDev = chs.getDevice(device.id);
    if(prevDev != undefined && prevDev.readings != undefined) {
        device.readings = prevDev.readings;
    }
    
    var err = chs.checkDevice(device);
    if(err != undefined) {
        res.send(400, err);
    } else {
        var id = chs.createDevice(device);
        if(id == undefined) {
            res.send(500, 'Server error.');
        } else {
            res.send(200,{'id':id});
        }
    }
}

var funcEdit = function(req, res, next) {
    var device = {};
    device.id = req.params.id;
    device.type = req.params.type;
    device.name = req.params.name;
    device.room = req.params.room;
    device.state = req.params.state;
    device.time = req.params.time;
    device.sort = req.params.sort;
    device.sort_dev = req.params.sort_dev;
    device.timer_type = req.params.timer_type;
    device.offset = req.params.offset;
    device.days = req.params.days;
    device.members = req.params.members;
    device.device = req.params.device;
    device.device_state = req.params.device_state;
    
    var prevDev = chs.getDevice(device.id);
    if(prevDev != undefined && prevDev.readings != undefined && prevDev.type != "timer") {
        device.readings = prevDev.readings;
    }
    
    //command = chs.editDevice(device);
    //if(command == undefined) {
    var err = chs.checkDevice(device);
    if(err != undefined) {
        res.send(400, err);
    } else {
        //chs.deleteDevice(device.id);
        chs.createDevice(device);
        res.send(200);
    }
}

var funcDelete = function(req, res, next) {
    chs.deleteDevice(req.params.id);
    res.send(200,'');
}

var funcState = function(req, res, next) {
    //TODO check States!
    if(req.params.id != undefined && req.params.state != undefined) {
        chs.setDeviceState(req.params.id, req.params.state);
        res.send(200,'');
    } else {
        res.send(500,'Error');
    }
}


module.exports = function(chsData) {
  chs = chsData;

  return {
    list: funcList,
    view: funcView,
    add: funcAdd,
    edit: funcEdit,
    del: funcDelete,
    state: funcState
  }
}
