"use strict";

var VERSION = '1.0.4'
var UPDATE = false;

//var mdns = require('mdns');

var config = require('./config');
var settingsData = require('./settings')();
var chs = require('./chs')(settingsData);
var restify = require('restify');
var bonjour = require('bonjour')();
var os = require('os');
var fs = require('fs');

//Route-Imports
var devices = require('./routes/devices')(chs);
var settings = require('./routes/settings')(settingsData);

String.prototype.startsWith = function(prefix) {
    return this.indexOf(prefix) === 0;
}

Number.prototype.pad = function(size) {
    var s = String(this);
    while (s.length < (size || 2)) {s = "0" + s;}
    return s;
}

var server = restify.createServer({
  name: 'COMMEO-API'
});

server.use(restify.bodyParser());
server.use(restify.authorizationParser());
server.use(restify.gzipResponse());
server.use(restify.CORS());
server.use(restify.fullResponse());
server.use(restify.queryParser());

server.get('/info', function(req, res, next) {
    var result = {
        server: 'COMMEO-API',
        version: VERSION,
        update: UPDATE
    };
    res.send(200, result);
});

//#####################################
// Devices
//#####################################
server.post('/devices', devices.add); //CREATE
server.get('/devices', devices.list); //READ
server.get('/devices/:id', devices.view); //READ
server.put('/devices/:id', devices.edit); //UPDATE
server.del('/devices/:id', devices.del); //DELETE

server.post('/devices/:id/state', devices.state);

//#####################################
// Settings
//#####################################
server.get('/settings', settings.listAll); //READ
server.get('/settings/:id', settings.list); //READ
server.del('/settings/:id', settings.del); //DELETE
server.put('/settings/:id', settings.write); //WRITE
server.post('/settings', settings.write); //WRITE


server.get('/export', function(req, res, next) {
    var result = {};
    result.version = "1.0";
    result.settings = settingsData.getSettings();
    result.devices = chs.getFullDevices();
    res.setHeader('content-type', 'application/octet-stream');
    res.send(200, JSON.stringify(result));
}); //Settings Export / File Download

server.get('/import', function(req, res, next) {
    var body = '<html><head><title>Update</title></head><body><form method="post" enctype="multipart/form-data"><input type="file" name="import"><input type="submit" name="Senden"></form></body></html>';

    res.writeHead(200, {
        'Content-Length': Buffer.byteLength(body),
        'Content-Type': 'text/html'
    });
    res.write(body);
    res.end();
});

server.get('/reset', function(req, res, next) {
        settingsData.importBackup({});
        chs.importBackup({});
        
        res.send(200, "");
        
        var process = require('child_process');
        var child= process.exec("echo \"systemctl restart chs\" | at now");
});


server.post('/import', function(req, res, next) {
    var file = req.files.import.path;
    var json = JSON.parse(fs.readFileSync(file, "utf8"));
    
    if(json != undefined && (json.settings != undefined || json.devices != undefined)) {
        settingsData.importBackup(json.settings);
        chs.importBackup(json.devices);
        
        res.send(200, "");
        
        var process = require('child_process');
        var child= process.exec("echo \"systemctl restart chs\" | at now");
    } else {
        res.send(500, "");
    }
}); //Settings Import / File Upload

server.get('/time', function(req, res, next) {
    var date = new Date();
    var result = {}
    result.time = Math.floor(date.valueOf()/1000);
    if(settingsData.getSettings().ntp == undefined) {
        settingsData.put("ntp", true);
    }
    result.ntp = settingsData.getSettings().ntp;
    result.timezone = fs.readFileSync("/etc/timezone", "utf8").trim();
    result.zonelist = [];

    var files = fs.readdirSync("/usr/share/zoneinfo/Europe");
    for (var i in files){
        var name = "Europe/" + files[i];
        result.zonelist.push(name);
    }

    var files = fs.readdirSync("/usr/share/zoneinfo/Asia");
    for (var i in files){
        var name = "Asia/" + files[i];
        result.zonelist.push(name);
    }

    res.send(200, result);
}); //Unix Timestamp

server.post('/time', function(req, res, next) {
    var date = new Date();
    date.setTime(req.params.time*1000);

    var timeString = date.getDate().pad() + "." + (date.getMonth()+1).pad() + "." + date.getFullYear() + " " + date.getHours().pad() + ":" + date.getMinutes().pad() + ":" + date.getSeconds().pad() + " " + req.params.timezone;
    
    if(req.params.ntp == true) {
        timeString += " ntp";
    }
    
    settingsData.put("ntp", req.params.ntp);
    
    console.log(timeString);

    var process = require('child_process');
    var child= process.exec("perl /opt/chs/setclock.pl " + timeString);

    res.send(200, "");
}); //Unix Timestamp {"time":123456789}


server.post('/update', function(req, res, next) {
    var process = require('child_process');
    var child= process.exec("echo \"sh /opt/chs/update.sh " + req.files.update.path + "\" | at now");
    res.header('Location', '/#/cfg/settings/updating')
    res.send(302, 'Ok');
}); //Toggle Update


server.get('/manual_update', function(req, res, next) {
    var body = '<html><head><title>Update</title></head><body><form method="post" enctype="multipart/form-data"><input type="file" name="update"><input type="submit" name="Senden"></form></body></html>';

    res.writeHead(200, {
        'Content-Length': Buffer.byteLength(body),
        'Content-Type': 'text/html'
    });
    res.write(body);
    res.end();
});

server.post('/manual_update', function(req, res, next) {
    var process = require('child_process');
    var child= process.exec("echo \"/opt/chs/update.sh " + req.files.update.path + "\" | at now");
    res.header('Location', '/#/cfg/settings/updating')
    res.send(302, 'Ok');
});


//#####################################
// Static content
//#####################################
server.get(/\//, restify.serveStatic({
  directory: '/opt/chs/webapp/',
  default: 'index.html',
  maxAge: 60
}));

server.listen(config.port, function() {
  console.log('%s listening at %s', server.name, server.url);
  //console.log(dataValidator.check("customer",null));
});

setTimeout(function(){
  console.log("bonjour started" + os.hostname());
  bonjour.publish({ name: 'selve-chs-' + settingsData.getId(), type: 'selve-chs', port: 80 })
  require('getmac').getMac(function(err, macAddress) {
    settingsData.put("mac", macAddress);
  });
}, 3000);
