var net = require('net');

var mockDevices = [];

var fBuffer = "";

var fhem = net.connect({host: "localhost", port: 7072}, function() {
	console.log('connected to fhem');
	fhem.on('data', function(data) {
		fBuffer = fBuffer + data;
		while(fBuffer.indexOf('\n') >= 0) {	
			line = fBuffer.substring(0, fBuffer.indexOf('\n'));
		
			fBuffer = fBuffer.substring(fBuffer.indexOf('\n')+1);
			
			sLine = line.split(' ');
			if(sLine.length == 3 && sLine[0] == "dummy") {
				dev = sLine[1].split('.');
				if(dev.length == 2 && dev[0] == "CGW") {
					if(sLine[2] == "up" || sLine[2] == "down" || sLine[2] == "stop") {
						runCommand(dev[1], sLine[2]);
					}
				}
			}
		}
	});
	fhem.on('end', function() {
	  console.log('disconnected from server');
	});
	
	fhem.write('\ninform on\n');

	initDevices();

	for(dev in mockDevices) {
		fetchDeviceInfo(mockDevices[dev]);
	}
});

function initDevices() {
	for(i = 0; i < 10; i++) {
		dev = {};
		dev.id = i;
		dev.type = 'shutter';
		dev.direction = 'stop';
		dev.targetPos = 0;
		dev.currentPos = 0;
		mockDevices.push(dev);
	}
}


function fetchDeviceInfo(device) {
	fhem.write("define CGW." + device.id + " dummy\n");
	fhem.write("attr CGW." + device.id + " webCmd up:down:stop:pos1:pos2\n");
	fhem.write("attr CGW." + device.id + " userattr " + device.type + "\n");
	fhem.write("setreading CGW." + device.id + " rfid " + device.id + "\n");
}

function getDirectionText(direction) {
	switch(direction) {
		case '0':
			return "unknown"
		case '1': 
			return "stopped"
		case '2':
			return "drive-up"
		case '3':
			return "drive-down"
	}
}

function getTypeText(t) {
	switch(t) {
		case '0':
			return "unknown";
		case '1':
			return "shutter";
		case '2':
			return "blind";
		case '3':
			return "awning";
		case '4':
			return "switch";
		case '5':
			return "dimmer";
	}
}

function timer(dev, direction, startCount, currentCount) {
	setTimeout(function() {
		if(direction == 'up') {
			dev.direction = 'drive-up'
			dev.targetPos = 0;
			dev.currentPos = (100 * (currentCount / startCount));
		}

		if(direction == 'down') {
			dev.direction = 'drive-down'
			dev.targetPos = 100;
			dev.currentPos = dev.targetPos - (dev.targetPos * (currentCount / startCount));
		}

		fhem.write("setreading CGW." + dev.id + " moving " + dev.direction + "\n");
		fhem.write("setreading CGW." + dev.id + " target " + dev.targetPos + "\n");
		fhem.write("setreading CGW." + dev.id + " current " + dev.currentPos + "\n");

		if(currentCount > 0) {
			timer(dev, direction, startCount, currentCount-1)
		} else {
			fhem.write("setreading CGW." + dev.id + " moving stopped\n");
		}
	}, 1000);
};

function runCommand(id, command) {
	for(idx in mockDevices) {
		if(mockDevices[idx].id == id) {
			timer(mockDevices[idx], command, 10, 10);
		}
	}
}
