var serialport = require('serialport'),// include the library

SerialPort = serialport.SerialPort, // make a local instance of it

portName = process.argv[2];

var types = [];

var myPort = new SerialPort(portName, {
   baudRate: 115200,
   // look for return and newline at the end of each data packet:
   //parser: serialport.parsers.readline("\n")
 });

myPort.on('open', showPortOpen);
myPort.on('data', saveLatestData);
myPort.on('close', showPortClose);
myPort.on('error', showError);

function showPortOpen() {
   console.log('port open. Data rate: ' + myPort.options.baudRate);
   myPort.write('<methodCall><methodName>selve.GW.device.getIDs</methodName></methodCall>\n');
}
 
function saveLatestData(data) {
   console.log(data);
}
 
function showPortClose() {
   console.log('port closed.');
}
 
function showError(error) {
   console.log('Serial port error: ' + error);
}



var parseString = require('xml2js').parseString;

var net = require('net');

var fBuffer = "";

var fhem = net.connect({host: "localhost", port: 7072}, function() {
	console.log('connected to fhem');
	fhem.write("define CGW dummy\n");
	fhem.write("attr CGW chsType commeo\n"); 

	fhem.on('data', function(data) {
		fBuffer = fBuffer + data;
		while(fBuffer.indexOf('\n') >= 0) {	
			line = fBuffer.substring(0, fBuffer.indexOf('\n'));
		
			fBuffer = fBuffer.substring(fBuffer.indexOf('\n')+1);
			
			parseFhemLine(line);
		}
	});
	fhem.on('end', function() {
	  console.log('disconnected from server');
	});
	
	fhem.write('\ninform on\n');
});

function parseFhemLine(line) {
    
    var cleanLine = line.split('fhem> ').join('');
    sLine = cleanLine.split(' ');
        
        console.log("FHEM-Line:" + cleanLine);
	if(sLine.length == 3 && sLine[0] == "dummy") {
		dev = sLine[1].split('.');
		if(dev.length == 2 && dev[0] == "CGW") {
			if(getCommandCode(sLine[2]) != -1) {
				runCommand(dev[1], sLine[2], 0);
			}
		} else if(dev.length == 1 && sLine[2] == "pair") {
			console.log("Jetzt sollte das Pairing gestartet werden...");
                        startScan();
		}
	} else if(sLine.length > 5 && sLine[0] == 'Global' && sLine[1] == 'global' && sLine[2] == 'ATTR' && sLine[4] == 'alias') {
		dev = sLine[3].split('.');
		if(dev.length == 2 && dev[0] == "CGW") {
			devId = dev[1];
			name = "";
			for(i = 5; i < sLine.length; i++) {
				name += sLine[i] + " ";
			}
			name = name.trim();
			setName(devId, name);
		}
	} else if(sLine.length == 4 && sLine[0] == 'Global' && sLine[1] == 'global' && sLine[2] == 'DELETED') {
            dev = sLine[3].split('.');
		if(dev.length == 2 && dev[0] == "CGW") {
                    console.log("Delete device: " + dev[1]);
			devId = dev[1];
			deleteDevice(devId);
		}
        } else if(sLine.length == 4 && sLine[0] == "dummy") {
		dev = sLine[1].split('.');
		if(dev.length == 2 && dev[0] == "CGW") {
			if(getCommandCode(sLine[2]) != -1) {
				runCommand(dev[1], sLine[2], sLine[3]);
			}
		}
	}
}

var resTagStart = "<methodResponse>";
var resTagEnd = "</methodResponse>";

var callTagStart = "<methodCall>";
var callTagEnd = "</methodCall>";

var buffer = "";

function saveLatestData(data) {
	buffer = buffer + data;
	if(buffer.indexOf(resTagStart) >= 0 && buffer.indexOf(resTagEnd) >= 0) {
		
		startPos = buffer.indexOf(resTagStart);
		endPos = buffer.indexOf(resTagEnd);
		
		xml = buffer.substring(startPos, endPos + resTagEnd.length);
		
		buffer = buffer.substring(endPos + resTagEnd.length);

		parseXML(xml);
		console.log("Found xml: " + xml);
	}

	if(buffer.indexOf(callTagStart) >= 0 && buffer.indexOf(callTagEnd) >= 0) {

		startPos = buffer.indexOf(callTagStart);
		endPos = buffer.indexOf(callTagEnd);

		xml = buffer.substring(startPos, endPos + callTagEnd.length);

		buffer = buffer.substring(endPos + callTagEnd.length);

		parseXML(xml);
		console.log("Found xml: " + xml);
	}	
  //console.log(data.toString());
};

function parseXML(xml) {
	//console.log(xml);
	parseString(xml, function(err, result) {
		if(result == undefined) return;
		methodName = "";
		array = [];
		if(result.methodResponse != null) {
			array = result.methodResponse.array;
			
			if(array == null) return;
			
			methodName = array[0].string.shift();
		}

		if(result.methodCall != null) {
			array = result.methodCall.array;
			
			if(array == null) return;
			
			methodName = result.methodCall.methodName;
		}

		processMethodCall(methodName, array[0]);
		
	});

}

function processMethodCall(name, params) {
	
	console.log(methodName);
	console.log(params);
	
	if(name == "selve.GW.event.device") {
		id = params.int[0];
		direction = params.int[1];
		currentPos = Math.round(params.int[2]*100/65535);
		targetPos = Math.round(params.int[3]*100/65535);
		infoBits = params.int[4];
		// params.int[5];
		
		dirText = getDirectionText(direction, types[id]);
		s = "";
		
		if(types[id] == 'shutter' || types[id] == 'jalousie' || types[id] == 'marquee' || types[id] == undefined) {
			s += "setreading CGW." + params.int[0] + " moving " + dirText + "\n";
			s += "setreading CGW." + params.int[0] + " target " + targetPos + "\n";
			s += "setreading CGW." + params.int[0] + " current " + currentPos + "\n";
		
			s += "setreading CGW." + params.int[0] + " connection " + !((infoBits & 0x01) == 0x01) + "\n";
			s += "setreading CGW." + params.int[0] + " overload " + ((infoBits & 0x02) == 0x02) + "\n";
			s += "setreading CGW." + params.int[0] + " obstacle " + ((infoBits & 0x04) == 0x04) + "\n";
			s += "setreading CGW." + params.int[0] + " danger_signal " + ((infoBits & 0x08) == 0x08) + "\n";
			s += "setreading CGW." + params.int[0] + " taughtin_lost " + ((infoBits & 0x10) == 0x10) + "\n";
			s += "setreading CGW." + params.int[0] + " automatic " + ((infoBits & 0x20) == 0x20) + "\n";
		} else if(types[id] == 'switch' || types[id] == 'night-switch') {
			s += "setreading CGW." + params.int[0] + " current " + dirText + "\n";
		}
		
		fhem.write(s);
		fhem.write("\n");
	}
	
	if(name == "selve.GW.device.getIDs") {
		bitmask = new Buffer(params.base64[0], 'base64').toString('binary');
		
		var bytes = [];
		for (var i = 0; i < bitmask.length; ++i)
		{
			bytes.push(bitmask.charCodeAt(i));
		}
		//console.log(typeof bytes);
		
		ids = [];
		count = 0;
		for(idx in bytes) {
			if((bytes[idx] & 0x01) == 0x01) {
				ids.push(count * 8 + 0);
			}
			if((bytes[idx] & 0x02) == 0x02) {
				ids.push(count * 8 + 1);
			}
			if((bytes[idx] & 0x04) == 0x04) {
				ids.push(count * 8 + 2);
			} 
			if((bytes[idx] & 0x08) == 0x08) {
				ids.push(count * 8 + 3);
			}
			if((bytes[idx] & 0x10) == 0x10) {
				ids.push(count * 8 + 4);
			}
			if((bytes[idx] & 0x20) == 0x20) {
				ids.push(count * 8 + 5);
			}
			if((bytes[idx] & 0x40) == 0x40) {
				ids.push(count * 8 + 6);
			}
			if((bytes[idx] & 0x80) == 0x80) {
				ids.push(count * 8 + 7);
			}
			count++;
		}
		console.log(ids);
                if(ids.length != 0) {
                    fetchDeviceInfo(ids, 1000);
                }
	}
	
	if(name == "selve.GW.device.getInfo") {
		id = params.int[0];
		rfid = params.int[1];
		type = getTypeText(params.int[2]);
		saveflag = params.int[3];
		lable = params.string[0];
		
		types[id] = type;

		console.log(id + " " + rfid + " " + type + " " + saveflag + " " + lable);
		
		s = "define CGW." + id + " dummy\n";
		s += "attr CGW." + id + " alias " + lable + "\n";
		s += "attr CGW." + id + " chsType " + type + "\n";
		s += "setreading CGW." + id + " rfid " + rfid + "\n";
		
		fhem.write(s);
	}
	
	if(name == "selve.GW.device.scanResult") {
            status = params.int[0];
            count = params.int[1];
            bitmask = new Buffer(params.base64[0], 'base64').toString('binary');
            
            if(status == 3) {
                processScanResult(bitmask);
            }
        }
}

function processScanResult(bitmask) {
    var bytes = [];
    for (var i = 0; i < bitmask.length; ++i)
    {
        bytes.push(bitmask.charCodeAt(i));
    }
    
    ids = [];
    count = 0;
    for(idx in bytes) {
        if((bytes[idx] & 0x01) == 0x01) {
            ids.push(count * 8 + 0);
        }
        if((bytes[idx] & 0x02) == 0x02) {
            ids.push(count * 8 + 1);
        }
        if((bytes[idx] & 0x04) == 0x04) {
            ids.push(count * 8 + 2);
        } 
        if((bytes[idx] & 0x08) == 0x08) {
            ids.push(count * 8 + 3);
        }
        if((bytes[idx] & 0x10) == 0x10) {
            ids.push(count * 8 + 4);
        }
        if((bytes[idx] & 0x20) == 0x20) {
            ids.push(count * 8 + 5);
        }
        if((bytes[idx] & 0x40) == 0x40) {
            ids.push(count * 8 + 6);
        }
        if((bytes[idx] & 0x80) == 0x80) {
            ids.push(count * 8 + 7);
        }
        count++;
    }
    console.log(ids);
    if(ids.length != 0) {
        saveDevices(ids, 1000);
    }
}

function saveDevices(devices, sleep) {
	setTimeout(function() {
		device = devices.pop();
		s = '<methodCall><methodName>selve.GW.device.save</methodName><array><int>' + device + '</int></array></methodCall>\n';
		console.log(s);
		myPort.write(s);
                setTimeout(function() {
                    s = '<methodCall><methodName>selve.GW.device.getInfo</methodName><array><int>' + device + '</int></array></methodCall>\n';
                    console.log(s);
                    myPort.write(s);
                    if(devices.length > 0) {
			saveDevices(devices, sleep);
                    }
                }, sleep);
	}, sleep);
}

function fetchDeviceInfo(devices, sleep) {
	setTimeout(function() {
		device = devices.pop();
		s = '<methodCall><methodName>selve.GW.device.getInfo</methodName><array><int>' + device + '</int></array></methodCall>\n';
		console.log(s);
		myPort.write(s);
		if(devices.length > 0) {
			fetchDeviceInfo(devices, sleep);
		}
	}, sleep);
}

function fetchDeviceInfo(devices, sleep) {
	setTimeout(function() {
		device = devices.pop();
		s = '<methodCall><methodName>selve.GW.device.getInfo</methodName><array><int>' + device + '</int></array></methodCall>\n';
		console.log(s);
		myPort.write(s);
		if(devices.length > 0) {
			fetchDeviceInfo(devices, sleep);
		}
	}, sleep);
}

function getDirectionText(direction, type) {
	if(type == 'shutter' || type == 'jalousie' || type == 'marquee' || type == undefined) {
		switch(direction) {
			case '0':
				return "unknown"
			case '1': 
				return "stopped"
			case '2':
				return "drive-up"
			case '3':
				return "drive-down"
		}
	} else {
		switch(direction) {
                        case '0':
                                return "unknown"
                        case '1':
                                return "off"
                        case '2':
                                return "on"
                        case '3':
                                return "on"
                }
	}
}

function getTypeText(t) {
	switch(t) {
		case '0':
			return "unknown";
		case '1':
			return "shutter";
		case '2':
			return "jalousie";
		case '3':
			return "marquee";
		case '4':
			return "switch";
		case '5':
			return "dimmer";
		case '6':
			return "night-switch";
		case '7':
			return "twilight-switch";
		case '8':
			return "heating";
		case '9':
			return "cooling";
		case '10':
			return "day-switch";
		case '11':
			return "gateway";
                default:
                    return "unknown";
	}
}

function getCommandCode(command) {
	switch(command) {
		case 'driveUp':
		case 'on':
			return 1;
		case 'driveDown': return 2;
		case 'stop':
		case 'off':
			return 0;
                case 'drivePos1': return 3;
                case 'savePos1': return 4;
                case 'drivePos2': return 5;
                case 'savePos2': return 6;
                case 'drivePos': return 7;
                case 'stepUp': return 8;
                case 'stepDown': return 9;
                case 'autoOn': return 10;
                case 'autoOff': return 11;
                default: return -1;
	}
}

function runCommand(id, command, pos) {
        pos = Math.round(pos * 65535/100);
	s = '<methodCall><methodName>selve.GW.command.device</methodName><array>';
	s += '<int>' + id + '</int>';
	s += '<int>' + getCommandCode(command) + '</int>';
	s += '<int>1</int>';
        s += '<int>' + pos + '</int>';
	s += '</array></methodCall>\n';
	myPort.write(s);
        
        console.log("#### " + s);
}

function setName(id, name) {
	s = '<methodCall><methodName>selve.GW.device.setLabel</methodName><array>';
	s += '<int>' + id + '</int>';
	s += '<string>' + name + '</string>';
	s += '</array></methodCall>\n';
	myPort.write(s);
}

function deleteDevice(id) {
	s = '<methodCall><methodName>selve.GW.device.delete</methodName><array>';
	s += '<int>' + id + '</int>';
	s += '</array></methodCall>\n';
	myPort.write(s);
}

function startScan() {
	s = '<methodCall><methodName>selve.GW.device.scanStart</methodName></methodCall>\n';
	myPort.write(s);
}

