"use strict";

var parseString = require('xml2js').parseString;
var serialjs = require('serialport-js');
var fs = require('fs');

var chs;
var port;

var types = [];
var watchdog;
var runCommandTimeout;

var busy = false;
var comQueue = [];

function showPortOpen() {    
   console.log('port open.');
   port.send('<methodCall><methodName>selve.GW.service.getState</methodName></methodCall>\n')
   //myPort.write('<methodCall><methodName>selve.GW.device.getIDs</methodName></methodCall>\n');
}
 
function showPortClose() {
   console.log('port closed.');
}
 
function showError(error) {
   console.log('Serial port error: ' + error);
}

var resTagStart = "<methodResponse>";
var resTagEnd = "</methodResponse>";

var callTagStart = "<methodCall>";
var callTagEnd = "</methodCall>";

var faultTagStart = "<fault>";
var faultTagEnd = "</fault>";

var buffer = "";

function saveLatestData(data) {
    buffer = buffer + data;
    if(buffer.indexOf(resTagStart) >= 0 && buffer.indexOf(resTagEnd) >= 0) {
        
        var startPos = buffer.indexOf(resTagStart);
        var endPos = buffer.indexOf(resTagEnd);
        
        var xml = buffer.substring(startPos, endPos + resTagEnd.length);
        
        buffer = buffer.substring(endPos + resTagEnd.length);

        parseXML(xml);
        console.log("Found xml: " + xml);
    }

    if(buffer.indexOf(callTagStart) >= 0 && buffer.indexOf(callTagEnd) >= 0) {

        var startPos = buffer.indexOf(callTagStart);
        var endPos = buffer.indexOf(callTagEnd);
        
        var xml = buffer.substring(startPos, endPos + callTagEnd.length);
        
        buffer = buffer.substring(endPos + callTagEnd.length);
        
        parseXML(xml);
        //console.log("Found xml: " + xml);
    }
    
    if(buffer.indexOf(faultTagStart) >= 0 && buffer.indexOf(faultTagEnd) >= 0) {

        var startPos = buffer.indexOf(faultTagStart);
        var endPos = buffer.indexOf(faultTagEnd);
        
        var xml = buffer.substring(startPos, endPos + faultTagEnd.length);
        
        buffer = buffer.substring(endPos + faultTagEnd.length);
        
        //parseXML(xml);
        //console.log("Found xml: " + xml);
    }
    
    //console.log(data.toString());
};

function parseXML(xml) {
    //var buf = new Buffer(input, 'binary');
    //var xml = buf.toString('utf8');
    
    parseString(xml, function(err, result) {
        if(result == undefined) {
            
            console.log(xml);
            console.log(err);
            return;
        }
        var methodName = "";
        var array = [];
        if(result.methodResponse != null) {
            array = result.methodResponse.array;
            
            if(array == null) return;
            
            methodName = array[0].string.shift();
        }
        
        if(result.methodCall != null) {
            array = result.methodCall.array;
            
            if(array == null) return;
            
            methodName = result.methodCall.methodName;
        }
        
        processMethodCall(methodName, array[0]);
        
        runNextCommand();
    });

}

function processMethodCall(name, params) {
    console.log(name);
    console.log(params);
    
    if(name == "selve.GW.service.getState") {
        var state = params.int[0];
        if(state == 3) {
            //ready state
            var s = '<methodCall><methodName>selve.GW.device.getIDs</methodName></methodCall>\n';
            addCommandString(s);
        } else {
            var s = '<methodCall><methodName>selve.GW.service.getState</methodName></methodCall>\n';
            addCommandString(s);
        }
    }
    
    if(name == "selve.GW.event.device" || name == "selve.GW.device.getValues") {
        var id = "CGW." + params.int[0];
        var direction = params.int[1];
        var currentPos = Math.round(params.int[2]*100/65535);
        var targetPos = Math.round(params.int[3]*100/65535);
        var infoBits = params.int[4];
        // params.int[5];
        
        var dirText = getDirectionText(direction, types[id]);
        
        var device = chs.getDevice(id);
        if(device == undefined) {
            device = {};
            device.id = id;
        }
        
        if(device.readings == undefined) {
            device.readings = {};
        }
        
        //if(types[id] == 'shutter' || types[id] == 'jalousie' || types[id] == 'marquee' || types[id] == undefined) {
        device.readings.moving = dirText;
        device.readings.target = targetPos;
        device.readings.current = currentPos;
        device.readings.connection = !((infoBits & 0x01) == 0x01);
        device.readings.overload = ((infoBits & 0x02) == 0x02);
        device.readings.obstacle = ((infoBits & 0x04) == 0x04);
        device.readings.danger_signal = ((infoBits & 0x08) == 0x08);
        device.readings.taughtin_lost = ((infoBits & 0x10) == 0x10);
        device.readings.automatic = ((infoBits & 0x20) == 0x20);
        //} else {
        //    device.readings.moving = dirText;
        //}
    
        chs.createDevice(device);
    }
    
    if(name == "selve.GW.device.getIDs") {
        var bitmask = new Buffer(params.base64[0], 'base64').toString('binary');
        
        var bytes = [];
        for (var i = 0; i < bitmask.length; ++i)
        {
            bytes.push(bitmask.charCodeAt(i));
        }
        //console.log(typeof bytes);
        
        var ids = maskToIds(bytes);
        
        console.log(ids);
        if(ids.length != 0) {
            fetchDeviceInfo(ids, 1000);
        } else {
            var cgw = chs.getDevice("CGW");
            cgw.state = "ready";
            chs.createDevice(cgw);
        }
    }
    
    if(name == "selve.GW.device.getInfo") {
        var id = "CGW." + params.int[0];
        var rfid = params.int[1];
        var type = getTypeText(params.int[2]);
        var saveflag = params.int[3];
        var lable = params.string[0];
        
        types[id] = type;
        
        console.log(id + " " + rfid + " " + type + " " + saveflag + " " + lable);
        
        var device = chs.getDevice(id);
        if(device == undefined) {
            device = {};
        }
        
        device.id = id;
        device.name = lable;
        device.type = type;
        
        if(device.readings == undefined) {
            device.readings = {};
        }
        
        device.readings.rfid = rfid;
        
        chs.createDevice(device)
    }
    
    if(name == "selve.GW.device.scanResult") {
        var status = params.int[0];
        var count = params.int[1];
        var bitmask = new Buffer(params.base64[0], 'base64').toString('binary');
        
        if(status == 3) {
            processScanResult(bitmask);
        }
    }

}

function processScanResult(bitmask) {
    var bytes = [];
    for (var i = 0; i < bitmask.length; ++i)
    {
        bytes.push(bitmask.charCodeAt(i));
    }
    
    var ids = maskToIds(bytes);
    
    console.log(ids);
    if(ids.length != 0) {
        saveDevices(ids, 1000);
    } else {
        var cgw = chs.getDevice("CGW");
        cgw.state = "ready";
        chs.createDevice(cgw);
    }
}


///////////////////////////////////////////////////////////////////////////////
// Device Methods
///////////////////////////////////////////////////////////////////////////////

function saveDevices(devices, sleep) {
    setTimeout(function() {
        var device = devices.pop();
        var s = '<methodCall><methodName>selve.GW.device.save</methodName><array><int>' + device + '</int></array></methodCall>\n';
        console.log("Run: " + s);
        port.send(s);
        setTimeout(function() {
            var s = '<methodCall><methodName>selve.GW.device.getInfo</methodName><array><int>' + device + '</int></array></methodCall>\n';
            console.log("Run: " + s);
            port.send(s);
            setTimeout(function() {
                s = '<methodCall><methodName>selve.GW.device.getValues</methodName><array><int>' + device + '</int></array></methodCall>\n';
                console.log("Run: " + s);
                port.send(s);
                if(devices.length > 0) {
                    saveDevices(devices, sleep);
                } else {
                    var cgw = chs.getDevice("CGW");
                    cgw.state = "ready";
                    chs.createDevice(cgw);
                }
            }, sleep);
        }, sleep);
    }, sleep);
}

function fetchDeviceInfo(devices, sleep) {
    setTimeout(function() {
        var device = devices.pop();
        var s = '<methodCall><methodName>selve.GW.device.getInfo</methodName><array><int>' + device + '</int></array></methodCall>\n';
        console.log("Run: " + s);
        port.send(s);
        if(devices.length > 0) {
            fetchDeviceInfo(devices, sleep);
        } else {
            var cgw = chs.getDevice("CGW");
            cgw.state = "ready";
            chs.createDevice(cgw);
        }
    }, sleep);
}

var funcRunCommand = function(id, command, pos, automatic) {
    if(pos == 'NaN') return;
    
    if(command == "drivePos" && (pos == 0 || pos == 100)) {
        if(pos == 0) {
            command = "driveUp"
        } else {
            command = "driveDown";
        }
        pos = 0;
    }
    
    pos = Math.round(pos * 65535/100);
    var s = '<methodCall><methodName>selve.GW.command.device</methodName><array>';
    s += '<int>' + id.split(".")[1] + '</int>';
    s += '<int>' + getCommandCode(command) + '</int>';
    if(automatic) {
        s += '<int>2</int>';
    } else {
        s += '<int>1</int>';
    }
    s += '<int>' + pos + '</int>';
    s += '</array></methodCall>\n';
        
    addCommandString(s);
}

var funcRunGroupCommand = function(ids, command, pos, automatic) {
    if(pos == 'NaN') return;
    
    var mask = Uint8ToBase64(new Uint8Array(idsToMask(ids)));
    
    pos = Math.round(pos * 65535/100);
    
    var s = '<methodCall><methodName>selve.GW.command.groupMan</methodName><array>';
    s += '<int>' + getCommandCode(command) + '</int>';
    if(automatic) {
        s += '<int>2</int>';
    } else {
        s += '<int>1</int>';
    }
    s += '<base64>' + mask + '</base64>';
    s += '<int>' + pos + '</int>';
    s += '</array></methodCall>\n';
    addCommandString(s);
}


var funcUpdateName = function(id, name) {
    var s = '<methodCall><methodName>selve.GW.device.setLabel</methodName><array>';
    s += '<int>' + id.split(".")[1] + '</int>';
    s += '<string>' + name + '</string>';
    s += '</array></methodCall>\n';
    addCommandString(s);
}

var funcDeleteDevice = function(id) {
    var s = '<methodCall><methodName>selve.GW.device.delete</methodName><array>';
    s += '<int>' + id.split(".")[1] + '</int>';
    s += '</array></methodCall>\n';
    addCommandString(s);
}

var funcStartScan = function() {
    var cgw = chs.getDevice("CGW");
    cgw.state = "scan";
    chs.createDevice(cgw);
    var s = '<methodCall><methodName>selve.GW.device.scanStart</methodName></methodCall>\n';
    addCommandString(s);
}


///////////////////////////////////////////////////////////////////////////////
// Internal Methods
///////////////////////////////////////////////////////////////////////////////


function addCommandString(cmd) {
    comQueue.push(cmd);
    if(!busy) {
        runNextCommand();
    }
}

function runNextCommand() {
    busy = true;
    if(runCommandTimeout == undefined) {
        runCommandTimeout = setTimeout(function() {
            if(comQueue.length > 0) {
                clearTimeout(watchdog);
                watchdog = setTimeout(function(){ busy = false; runNextCommand(); console.log("watchdog fired!") }, 20000);
                var s = comQueue.pop();
                console.log("Run: " + s);
                port.send(s);
            } else {
                clearTimeout(watchdog);
                busy = false;
            }
            runCommandTimeout = undefined;
        }, 250);
    }
}

function getDirectionText(direction, type) {
    if(type == 'shutter' || type == 'jalousie' || type == 'marquee' || type == undefined) {
        switch(direction) {
            case '0':
                return "unknown"
            case '1': 
                return "stopped"
            case '2':
                return "drive-up"
            case '3':
                return "drive-down"
        }
    } else {
        switch(direction) {
            case '0':
                return "unknown"
            case '1':
                return "off"
            case '2':
                return "on"
            case '3':
                return "on"
            }
    }
}

function getTypeText(t) {
    switch(t) {
        case '0':
            return "unknown";
        case '1':
            return "shutter";
        case '2':
            return "jalousie";
        case '3':
            return "marquee";
        case '4':
            return "switch";
        case '5':
            return "dimmer";
        case '6':
            return "night-switch";
        case '7':
            return "twilight-switch";
        case '8':
            return "heating";
        case '9':
            return "cooling";
        case '10':
            return "day-switch";
        case '11':
            return "gateway";
        default:
            return "unknown";
    }
}

function getCommandCode(command) {
    switch(command) {
        case 'driveUp':
        case 'on':
            return 1;
        case 'driveDown': return 2;
        case 'stop':
        case 'off':
            return 0;
        case 'drivePos1': return 3;
        case 'savePos1': return 4;
        case 'drivePos2': return 5;
        case 'savePos2': return 6;
        case 'drivePos': return 7;
        case 'stepUp': return 8;
        case 'stepDown': return 9;
        case 'autoOn': return 10;
        case 'autoOff': return 11;
        default: return -1;
    }
}

function maskToIds(mask) {
    var ids = [];
    var count = 0;
    for(var idx in mask) {
        if((mask[idx] & 0x01) == 0x01) {
            ids.push(count * 8 + 0);
        }
        if((mask[idx] & 0x02) == 0x02) {
            ids.push(count * 8 + 1);
        }
        if((mask[idx] & 0x04) == 0x04) {
            ids.push(count * 8 + 2);
        } 
        if((mask[idx] & 0x08) == 0x08) {
            ids.push(count * 8 + 3);
        }
        if((mask[idx] & 0x10) == 0x10) {
            ids.push(count * 8 + 4);
        }
        if((mask[idx] & 0x20) == 0x20) {
            ids.push(count * 8 + 5);
        }
        if((mask[idx] & 0x40) == 0x40) {
            ids.push(count * 8 + 6);
        }
        if((mask[idx] & 0x80) == 0x80) {
            ids.push(count * 8 + 7);
        }
        count++;
    }
    
    return ids;
}

function idsToMask(ids) {
    var mask = [0,0,0,0,0,0,0,0];
    
    for(var idx in ids) {
        var id = parseInt(ids[idx].split(".")[1]);
        var mIdx = ~~(id / 8);
        
        var b = mask[mIdx];
        b = b | 1<<(id%8);
        mask[mIdx] = b;
    }
    
    return mask;
}

function Uint8ToBase64(u8){
  return new Buffer(u8).toString('base64');
}

function startSerial(p){
    port = p;
    port.on('data', saveLatestData);
    port.on('close', showPortClose);
    port.on('error', showError);
    showPortOpen();
}

module.exports = function(chsData) {
    chs = chsData;
    
    var cgw = chs.getDevice("CGW");
    if(cgw == undefined) {
        cgw = {};
        cgw.id = "CGW";
        cgw.type = "gateway";
    }
    
    if (fs.existsSync('/dev/ttyUSB0')) {
        cgw.state = "init";
        chs.createDevice(cgw);
        serialjs.open(
        '/dev/ttyUSB0',
        startSerial,
        ''
    );
    } else {
        cgw.state = "na";
        chs.createDevice(cgw);
    }

    return {
        startScan: funcStartScan,
        runCommand: funcRunCommand,
        runGroupCommand: funcRunGroupCommand,
        updateName: funcUpdateName,
        delete: funcDeleteDevice
    }
}
