import { Component, OnInit, ElementRef, ViewChild } from "@angular/core";
import { Response } from "@angular/http";
import { Router } from "@angular/router";
import { Translation, LocaleService, TranslationService } from "angular-l10n";
import { ChsService } from "./chs.service";
import { AuthService } from "./auth.service";
import { SettingsService } from "./settings/settings.service";
import { Version } from "./version";
import "bootstrap";
import * as moment from "moment";
import { Device } from "./devices/device";
import { ILanguage } from "./settings/setting.models";

@Component({
	moduleId: module.id,
	selector: "commeo-app",
	templateUrl: "app.component.html"
})
export class AppComponent implements OnInit {
	@ViewChild("navMain") private _elNavMain: ElementRef;
	private _navMain: JQuery;
	private _stickNA: boolean = false;


	constructor(
		public locale: LocaleService, public translation: TranslationService,
		private _chsService: ChsService, private _authService: AuthService, private _settingsService: SettingsService,
		private _router: Router) {
		let curLang: string = this.locale.getCurrentLanguage();
		let lang: ILanguage = this._settingsService.getLanguage(curLang);
		if (!lang) {
			lang = this._settingsService.getBrowserLanguage();
		}

		if (lang) {
			this.locale.setCurrentLanguage(lang.iso);
		}

		/*
		this.Translation.useLocalStorage();
		this.Translation.addLanguages(["de", "en", "fr", "nl", "po", "cz"]);
		this.Translation.definePreferredLocale("en", "EN");
		this.Translation.definePreferredCurrency("EUR");
		this.Translation.setCurrentLocale("en", "EN");
		this.translation.translationProvider("./resources/Translation-");
		this.translation.updateTranslation();*/
		moment.locale("de");
	}

	public title = "commeo Home Server";
	public version: Version = { version: "", server: "", update: false };
	public errMsg: string;

	private _menu: JQuery;

	selectLocale(language: string, country: string, currency: string): void {
		this.locale.setCurrentLanguage(language);
		this.locale.setCurrentCurrency(currency);
	}

	getVersion() {
		this._chsService.getVersion().subscribe(
			ver => this.version = ver,
			err => this.errMsg = err
		);
	}

	ngOnInit() {
		if (!this._chsService.GotVersionInfo)
			this.getVersion();
		else
			this.version = this._chsService.VersionInfo;
		this._navMain = jQuery(this._elNavMain.nativeElement);
		this._chsService.onError.subscribe((res: Response) => { this.onChsServiceError(res); });
		this._chsService.getGateway().subscribe( ( gw: Device) => { this._stickNA = (gw.state==="na"); }  );
	}

	onChsServiceError(res: Response) {
		if (res.status === 0) {
			this._chsService.showMessageBox(this.translation.translate("SERVERNOTREACHABLE"));
		}
		else {
			this._chsService.showMessageBox(this.translation.translate("SERVERERROR") + ": (" + res.status + ") " + res.statusText);
		}
	}

	onLogout() {
		this._authService.logout();
		this._navMain.collapse("hide");
		this._router.navigate(["/favorites"]);
	}

	onMenuClick(event: any) {
		this._navMain.collapse("hide");
	}
}


