import { APP_INITIALIZER, Injectable, NgModule } from "@angular/core";
import { BrowserModule } from "@angular/platform-browser";
import { FormsModule } from "@angular/forms";
import { HttpModule } from "@angular/http";
import { TranslationModule, LocaleService, TranslationService } from "angular-l10n";
import { DriveModule } from "./drive/drive.module";
import { DevicesModule } from "./devices/devices.module";
import { GroupsModule } from "./groups/groups.module";
import { RoomsModule } from "./rooms/rooms.module";
import { ScenesModule } from "./scenes/scenes.module";
import { UpdateModule } from "./update/update.module";
import { SettingsModule } from "./settings/settings.module";
import { routing, appRoutingProviders } from "./app.routing";

import { ChsService } from "./chs.service";
import { AuthGuard } from "./auth-guard.service";
import { AuthService } from "./auth.service";
import { TranslateService } from "./translation.service";
import { SettingsService } from "./settings/settings.service";
import { AppComponent } from "./app.component";
import { ILanguage } from "./settings/setting.models";


/**
 * Advanced initialization.
 * 
 * With these settings, translation file will be loaded before the app.
 */
@Injectable()
export class LocalizationConfig {

  constructor(public locale: LocaleService, public translation: TranslationService, public settings: SettingsService) { }

  load(): Promise<any> {
    let languages: string[] = ["de", "en", "fr", "nl", "pl", "cs"];
    let startLang: ILanguage = this.settings.getBrowserLanguage();
    if (languages.findIndex((lang: string) => startLang.iso === lang) ===-1) {
      startLang=this.settings.LANGUAGES[1];
    }

    this.locale.addConfiguration()
      .useLocalStorage()
      .addLanguages(languages)
      .defineDefaultLocale(startLang.iso, startLang.country)
      .defineCurrency("EUR");
    this.locale.init();
    //this.locale.setDefaultLocale(startLang.iso, startLang.country);
    this.translation.addConfiguration().addProvider("./resources/locale_");
    this.translation.init();

    var promise: Promise<any> = new Promise((resolve: any) => {
      this.translation.translationChanged.subscribe(() => {
        resolve(true);
      });
    });

    return promise;
  }
}

/**
 * Aot compilation requires a reference to an exported function.
 */
export function initLocalization(localizationConfig: LocalizationConfig): Function {
  return () => localizationConfig.load();
}

@NgModule({
  imports: [
    TranslationModule.forRoot(),
    BrowserModule, FormsModule, HttpModule, DriveModule,
    DevicesModule, GroupsModule, RoomsModule, ScenesModule, UpdateModule, SettingsModule,
    routing
  ],
  declarations: [
    AppComponent
  ],
  providers: [
    LocalizationConfig,
    {
      provide: APP_INITIALIZER,
      useFactory: initLocalization,
      deps: [LocalizationConfig],
      multi: true
    },
    ChsService, AuthService, TranslateService, AuthGuard, appRoutingProviders],
  bootstrap: [AppComponent]
})
export class AppModule { }