import { Injectable, EventEmitter } from "@angular/core";
import { Http, Response, RequestOptions, Headers } from "@angular/http";
import { Version } from "./version";
import { WithId, Device, Member, Reading } from "./devices/device";
import { SettingsRead } from "./settings/setting";
import { SettingTime } from "./settings/setting.models";
import { Observable } from "rxjs/Observable";
import { Subscription } from "rxjs/Subscription";
import { maxSort } from "./utils";
//import {AsyncSubject} from 'rxjs/AsyncSubject';
//import 'rxjs/add/operator/merge';
//import 'rxjs/add/operator/concatAll';
import "rxjs/add/operator/catch";
import "rxjs/add/operator/map";
import "rxjs/add/operator/delay";
import "rxjs/add/operator/concat";
import "rxjs/add/operator/do";
import "rxjs/add/operator/repeat";
import "rxjs/add/operator/merge";
import "rxjs/add/operator/publish";
import "rxjs/add/operator/retry";
import "rxjs/observable/range";
import { ErrorObservable } from "rxjs/observable/ErrorObservable";
import "rxjs/observable/throw";

export class DevState {
	state: number;
	driving: boolean;
	auto: boolean;

	static equals(d1: DevState, d2: DevState): boolean {
		return d1.state === d2.state && d1.driving === d2.driving && d1.auto === d2.auto;
	}
}

export interface IStateChange {
	devId: string;
	oldState: DevState;
	newState: DevState;
}

@Injectable()
export class ChsService {
	public static FAVORITES = "Favorites";
	public static VERSIONSTORE = "ApiVersion";

	//private _url = 'http://debian8.fritz.box:8080/';
	//private _url = "http://192.168.188.20:80/";
	//private _url = 'http://127.0.0.1:8080/';
	//private _url='http://localhost:8080/';
	private _url = location.origin + "/";
	public get Url(): string {
		return this._url;
	}

	onError = new EventEmitter<Response>();
	stateChanged = new EventEmitter<IStateChange[]>();
	onGotSettings = new EventEmitter();
	onPairingFinished = new EventEmitter();
	private stateChangingSubscription: Subscription;
	private stateChangingSubscriptionCount: number = 0;
	private stateChanging: Observable<IStateChange[]>;

	constructor(private http: Http) {
		this.stateChanging = this.http.get(this._url + "devices?filter=real", )
			.catch((err, caught) => this.handleError(err, caught, this))
			.map(res => this.mapToChanges(res))
			.delay(1000).repeat();
		//.do(sc => { console.debug("State changes: " + sc ? sc.length.toString() : "undef"); }).repeat();
	}

	private mapToChanges(res: any): IStateChange[] {
		let service = this;
		let devs = <Device[]>res.json();
		var changes: IStateChange[] = [];
		devs.forEach(dev => {
			if (dev.type !== "group" && dev.type !== "scene" && dev.type !== "timer" &&
				dev.type !== "room" && dev.type !== "gateway" && dev.readings) {
				ChsService.convertReadingsToMap(dev);
				let readings = (<Map<string, string | number | boolean>>dev.readings);
				var moving: string = <string>readings.get("moving");

				let oldDev = service._mapRealDevs.get(dev.id);
				var oldDevState: DevState = { state: null, driving: null, auto: null };
				let oldReadings = (<Map<string, string | number | boolean>>oldDev.readings);
				var oldMoving: string = "";
				if (oldReadings) {
					oldMoving = <string>oldReadings.get("moving");
					oldDevState.driving = !(!oldMoving || oldMoving === "stopped");
					oldDevState.auto = <boolean>oldReadings.get("automatic");
					oldDevState.state = <number>oldReadings.get("current");
				}
				var sc: IStateChange = <IStateChange>{
					devId: dev.id,
					newState: { state: readings.get("current"), driving: !(!moving || moving === "stopped"), auto: readings.get("automatic") },
					oldState: oldDevState
				};

				if (!DevState.equals(sc.oldState, sc.newState)) {
					changes.push(sc);
					oldDev.readings = readings;
					service._mapRealDevs.set(oldDev.id, oldDev);
				}
			}
		});
		if (changes.length > 0) {
			service.stateChanged.emit(changes);
		}
		return changes;
	}

	private _mapDevs: Map<string, Device> = new Map<string, Device>();
	private _mapRealDevs: Map<string, Device> = new Map<string, Device>();
	private _mapGroups: Map<string, Device> = new Map<string, Device>();
	private _mapScenes: Map<string, Device> = new Map<string, Device>();
	private _mapTimers: Map<string, Device> = new Map<string, Device>();
	private _mapRooms: Map<string, Device> = new Map<string, Device>();
	private _mapSettings: Map<string, any> = new Map<string, any>();
	private _setFavorites: Set<string> = new Set<string>();
	private _gateway: Device = { type: "gateway", id: "" };

	public msgboxtext: string;

	get Gateway(): Device {
		return this._gateway;
	}

	get Groups(): Map<string, Device> {
		return this._mapGroups;
	}

	get Scenes(): Map<string, Device> {
		return this._mapScenes;
	}

	get Timers(): Map<string, Device> {
		return this._mapTimers;
	}

	get Rooms(): Map<string, Device> {
		return this._mapRooms;
	}

	get RealDevs(): Map<string, Device> {
		return this._mapRealDevs;
	}

	get Devices(): Map<string, Device> {
		return this._mapDevs;
	}

	get VersionInfo(): Version {
		let versionInfo: Version = <Version>JSON.parse(sessionStorage.getItem(ChsService.VERSIONSTORE));
		return versionInfo;
	}

	get GotVersionInfo(): boolean {
		return sessionStorage.getItem(ChsService.VERSIONSTORE) !== undefined;
	}

	private static convertReadingsToMap(dev: Device) {
		if (!dev.readings) { return; }
		let readings: Reading[] = (<Reading[]>dev.readings);
		dev.readings = new Map<string, string>();
		Object.keys(readings).forEach(key => {
			(<Map<string, string>>dev.readings).set(key, readings[key]);
		});
	}

	private fillMap(devs: Device[]) {
		this._mapDevs.clear();
		this._mapRealDevs.clear();
		this._mapGroups.clear();
		this._mapScenes.clear();
		devs.forEach(dev => {
			if (dev.type === "group") {
				if (!dev.members)
					dev.members = <string[]>[];
				this._mapGroups.set(dev.id, dev);
			}
			else if (dev.type === "scene") {
				if (!dev.members)
					dev.members = <Member[]>[];
				this._mapScenes.set(dev.id, dev);
			}
			else if (dev.type === "timer")
				this._mapTimers.set(dev.id, dev);
			else if (dev.type === "room")
				this._mapRooms.set(dev.id, dev);
			else if (dev.type === "gateway") {
				this._gateway = dev;
			}
			else {
				ChsService.convertReadingsToMap(dev);
				dev.group = "nogroup";
				this._mapRealDevs.set(dev.id, dev);
			}
			this._mapDevs.set(dev.id, dev);
		});

		/*	this._mapRealDevs.forEach(dev => {
				dev.group = "nogroup";
				this._mapGroups.forEach(group => {
					(<string[]>group.members).forEach(member => {
						if (dev.id == member) { dev.group = group.id; }
					})
				})
				this._mapScenes.forEach(scene => {
					(<Member[]>scene.members).forEach(member => {
						if (dev.id == member.id) { dev.scene = scene.id; }
					})
				})
			});*/
	}

	private delGroup(id: string) {
		if (this._mapGroups.delete(id)) {
			this._mapDevs.delete(id);
			this._mapRealDevs.forEach(dev => {
				if (dev.group === id) dev.group = "nogroup";
			});
		}
	}

	private delScene(id: string) {
		if (this._mapScenes.delete(id)) {
			this._mapDevs.delete(id);
			this._mapRealDevs.forEach(dev => {
				if (dev.device === id) dev.device = "noscene";
			});
		}
	}

	private delTimer(id: string) {
		this._mapTimers.delete(id);
		this._mapDevs.delete(id);
	}

	private delDevice(id: string) {
		if (this._mapDevs.delete(id)) {
			// TODO: Gerät aus Gruppen und Szenen löschen ???
		}
	}

	showMessageBox(strMsg: string) {
		this.msgboxtext = strMsg;
	}

	getVersion(): Observable<Version> {
		return this.http.get(this._url + "info").catch((err, caught) => this.handleError(err, caught, this))
			.map(res => <Version>res.json())
			.do(data => { sessionStorage.setItem(ChsService.VERSIONSTORE, JSON.stringify(data)); });
	}

	getVersionRaw(): Observable<Version> {
		return this.http.get(this._url + "info")
			.map(res => <Version>res.json())
			.catch((err, caught) => err);
	}

	pollServerVersion(cb: any): void {
		let obVersion: Observable<Response> = this.http.get(this._url + "info");

		let hotRepeat = new Observable().delay(10000).repeat().publish();


		//return obVersion.delay(10000).takeWhile((val, index) => !val.version);
		let safeRepeat = hotRepeat.concat(obVersion).retry();
		safeRepeat.subscribe();
		hotRepeat.connect();
	}


	getDevices(filter?: string) {
		let url = this._url + "devices";
		if (filter) { url += "?filter=" + filter }
		let obDevices = this.http.get(url).map(res => { var devs = <Device[]>res.json(); return devs })
			.do(devs => { this.fillMap(devs); });
		let obSettings = this.http.get(this._url + "settings")
			.map(res => { var settings = <SettingsRead>res.json(); return settings; })
			.do(settings => { this.storeSettings(settings); });

		return obSettings.merge(obDevices)
			.do(x => { }, err => { }, () => { this.applySettingsToDevs(); })
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	getGateway(): Observable<Device> {
		let obDevices = this.http.get(this._url + "devices?filter=gateway")
			.catch((err, caught) => this.handleError(err, caught, this))
			.map(res => {
				try { var devs = <Device[]>res.json(); this._gateway = devs[0]; return devs[0]; } catch (e) { return null; }
			})
			.do(dev => { this._gateway = dev; });
		return obDevices;
	}

	startGettingStateChange() {
		return this.http.get(this._url + "devices")
			.catch((err, caught) => this.handleError(err, caught, this))
			.map(res => { var devs = <Device[]>res.json(); return devs; })
			.do(devs => { this.fillMap(devs); })
			;
	}

	deleteGroup(id: string) {
		return this.http.delete(this._url + "devices/" + id).do(data => {
			console.log("deleted group " + id);
			this.delGroup(id);
		}).catch((err, caught) => this.handleError(err, caught, this));
	}

	deleteScene(id: string) {
		return this.http.delete(this._url + "devices/" + id).do(data => {
			console.log("deleted scene " + id);
			this.delScene(id);
		}).catch((err, caught) => this.handleError(err, caught, this));
	}

	deleteTimer(id: string) {
		return this.http.delete(this._url + "devices/" + id).do(data => {
			console.log("deleted timer " + id);
			this.delTimer(id);
		}).catch((err, caught) => this.handleError(err, caught, this));
	}

	deleteRoom(id: string): Observable<Response> {
		let obDel = [this.http.delete(this._url + "devices/" + id).do(data => {
			console.log("deleted room " + id);
			this._mapRooms.delete(id);
			this._mapDevs.delete(id);
		})];


		this._mapRealDevs.forEach(dev => {
			if (dev.room === id) {
				dev.room = null;
				obDel.push(this.writeDevice(dev));
			}
		});

		return this.mergeResp(obDel);
	}


	deleteDevice(id: string) {
		return this.http.delete(this._url + "devices/" + id).do(data => {
			console.log("deleted device " + id);
			this.delDevice(id);
		}).catch((err, caught) => this.handleError(err, caught, this));
	}

	private newDevice(dev: Device, devCB: (dev: Device) => void): Observable<Device> {
		let body = JSON.stringify(dev);
		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });

		return this.http.post(this._url + "devices", body, options)
			.catch((err, caught) => this.handleError(err, caught, this))
			.map(res => {
				var data = <WithId>res.json();
				dev.id = data.id;
				this._mapDevs.set(dev.id, dev);
				devCB(dev);
				return dev;
			})
			.do(data => console.log("NewDevice: " + data));
	}

	newTimer(sceneid: string): Observable<Device> {
		var nIdCount = 0;
		var newTimerId: string = "TIMER.";

		var setIds = new Set<string>(this._mapDevs.keys());
		var setNames = new Set<string>();

		this._mapDevs.forEach(dev => {
			setNames.add(dev.name);
		});

		while (setIds.has(newTimerId + nIdCount.toString()))
			++nIdCount;

		var dev: Device = {
			id: newTimerId + nIdCount.toString(), name: "", device: sceneid, device_state: "", type: "timer",
			days: "1111111", timer_type: "static", time: "12:00", state: "on", favorite: false, sort: maxSort(this._mapTimers) + 1
		};

		return this.newDevice(dev, d => {
			this._mapTimers.set(d.id, d);
		});
	}


	newScene(): Observable<Device> {
		var nIdCount = 0;
		var newSceneId: string = "SCENE.";
		var nNameCount = 0;
		var newName: string = "Neue Szene ";

		var setIds = new Set<string>(this._mapDevs.keys());
		var setNames = new Set<string>();

		this._mapDevs.forEach(dev => {
			setNames.add(dev.name);
		});

		while (setIds.has(newSceneId + nIdCount.toString()))
			++nIdCount;
		while (setNames.has(newName + nNameCount.toString()))
			++nNameCount;

		let dev = {
			id: newSceneId + nIdCount.toString(), name: newName + nNameCount.toString(), type: "scene",
			members: new Array<Member>(), favorite: false, sort: maxSort(this._mapScenes) + 1
		};

		return this.newDevice(dev, d => {
			this._mapScenes.set(d.id, d);
		});
	}

	removeFromScene(id: string, sceneid: string) {
		var dev: Device = this._mapRealDevs.get(id);
		if (!dev) return;

		var scene: Device = this._mapScenes.get(sceneid);
		if (!scene) return;

		scene.members = (<Member[]>scene.members).filter(value => value.id !== id);

		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });

		return this.http.put(this._url + "devices/" + sceneid, JSON.stringify(scene), options)
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	addToScene(id: string, sceneid: string, value: string | number) {
		var dev: Device = this._mapRealDevs.get(id);
		if (!dev) return;

		var scene: Device = this._mapScenes.get(sceneid);
		if (!scene) return;

		let index = (<Member[]>scene.members).findIndex(dev => dev.id === id);

		if (index < 0) {
			(<Member[]>scene.members).push({ id: id, value: value });
		}
		else {
			(<Member[]>scene.members)[index].value = value;
		}

		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });

		return this.http.put(this._url + "devices/" + sceneid, JSON.stringify(scene), options)
			.catch((err, caught) => this.handleError(err, caught, this));
	}


	newGroup(): Observable<Device> {
		var nIdCount = 0;
		var newGroupId: string = "GROUP.";
		var nNameCount = 0;
		var newName: string = "Neue Gruppe ";

		var setIds = new Set<string>(this._mapDevs.keys());
		var setNames = new Set<string>();

		this._mapDevs.forEach(dev => {
			setNames.add(dev.name);
		});

		while (setIds.has(newGroupId + nIdCount.toString()))
			++nIdCount;
		while (setNames.has(newName + nNameCount.toString()))
			++nNameCount;

		let dev = {
			id: newGroupId + nIdCount.toString(), name: newName + nNameCount.toString(), type: "group",
			members: new Array<string>(), favorite: false, sort: maxSort(this._mapGroups) + 1
		};

		return this.newDevice(dev, d => {
			this._mapGroups.set(d.id, d);
		});
	}

	getGroupState(id: string): string {
		let group = this._mapGroups.get(id);
		if (group && group.members && group.members.length > 0) {
			let devId = (<string[]>group.members)[0];
			let dev = this._mapRealDevs.get(devId);
			if (dev)
				return ChsService.getTypeState(dev.type);
		}

		return "";
	}

	getDeviceState(id: string): string {
		let dev = this._mapRealDevs.get(id);
		if (dev)
			return ChsService.getTypeState(dev.type);
		dev = this._mapGroups.get(id);
		if (dev)
			return this.getGroupState(id);
		dev = this._mapScenes.get(id);
		if (dev)

			return "";
	}

	static getTypeState(devType: string): string {
		if (devType === "dimmer" || devType === "shutter" || devType === "marquee" || devType === "jalousie")
			return "slider";
		else if (devType === "night-switch" || devType === "twighlight-switch" || devType === "day-switch")
			return "onoff";
		else if (devType === "scene")
			return "startstop";

		return "";
	}


	removeFromGroup(id: string, groupid: string) {
		var dev: Device = this._mapRealDevs.get(id);
		if (!dev) return;

		var group: Device = this._mapGroups.get(groupid);
		if (!group) return;

		group.members = (<string[]>group.members).filter(value => value !== id);

		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });

		return this.http.put(this._url + "devices/" + groupid, JSON.stringify(group), options)
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	addToGroup(id: string, groupid: string) {
		var dev: Device = this._mapRealDevs.get(id);
		if (!dev) return;

		var group: Device = this._mapGroups.get(groupid);
		if (!group) return;

		if (!(<string[]>group.members).find(dev => dev === id)) {
			(<string[]>group.members).push(id);
		}

		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });

		return this.http.put(this._url + "devices/" + groupid, JSON.stringify(group), options)
			.catch((err, caught) => this.handleError(err, caught, this));
	}


	changeGroup(id: string, newGroup: string) {
		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });
		var dev: Device = this._mapRealDevs.get(id);
		if (!dev) return;

		var devNewGroup: Device = null;
		if (newGroup !== "nogroup") {
			devNewGroup = this._mapGroups.get(newGroup);
		}
		var devOldGroup: Device = null;
		if (dev.group !== "nogroup") {
			devOldGroup = this._mapGroups.get(dev.group);
		}

		if (devNewGroup)
			(<string[]>devNewGroup.members).push(dev.id);
		if (devOldGroup) {
			var members = (<string[]>devOldGroup.members);
			devOldGroup.members = [];
			members.forEach(member => { if (member !== dev.id) (<string[]>devOldGroup.members).push(member); });
		}


		if (devOldGroup && devNewGroup) {
			return this.http.put(this._url + "devices/" + devOldGroup.id, JSON.stringify(devOldGroup), options).concat(
				this.http.put(this._url + "devices/" + devNewGroup.id, JSON.stringify(devNewGroup), options))
				.do(data => {
					dev.group = devNewGroup.id;
				})
				.catch((err, caught) => this.handleError(err, caught, this));
		}
		else if (devNewGroup) {
			return this.http.put(this._url + "devices/" + devNewGroup.id, JSON.stringify(devNewGroup), options)
				.do(data => {
					dev.group = devNewGroup.id;
				})
				.catch((err, caught) => this.handleError(err, caught, this));
		}
		else {
			return this.http.put(this._url + "devices/" + devOldGroup.id, JSON.stringify(devOldGroup), options).do(data => {
				dev.group = "nogroup";
			})
				.catch((err, caught) => this.handleError(err, caught, this));
		}
	}

	writeDevice(dev: Device): Observable<Response> {
		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });
		return this.http.put(this._url + "devices/" + dev.id, JSON.stringify(dev), options)
			.catch((err, caught) => this.handleError(err, caught, this));
		/* Hat nicht funktioniert: put ohne Body
		var gottenDevice : Device;
		return this.http.get(this._url+"devices/" + dev.id).do(data => { gottenDevice =<Device> data.json(); gottenDevice.name=dev.name; console.log("renamed " + JSON.stringify(gottenDevice)); })
			.concat(this.http.put(this._url+"devices/" + dev.id, JSON.stringify(gottenDevice), options)).catch(this.handleError);*/
	}

	mergeResp(ar: Observable<Response>[]): Observable<Response> {
		if (ar.length === 0) return null;
		if (ar.length === 1) return ar[0];
		//return Observable.merge(ar);
		/*return Observable.concatAll(ar);*/
		let result = ar[0].merge(ar[1]);
		for (let i = 2; i < ar.length; ++i) {
			result = result.merge(ar[i]);
		}

		return result;
	}


	writeDevices(devs: Device[]): Observable<Response> {
		if (devs.length === 0) return null;
		if (devs.length === 1)
			return this.writeDevice(devs[0]);

		var ar: Observable<Response>[] = [];
		devs.forEach(dev => {
			ar.push(this.writeDevice(dev));
		});

		return this.mergeResp(ar);
	}

	beginTeachIn(): Observable<any> {
		let observableGetState = this.getGateway().map(dev => dev.state);
		let observablePair = this.setDevState(this._gateway.id, "pair").map(x => "").do(x => { console.debug("begin pairing"); });

		return observablePair.concat(observableGetState.delay(1000).repeat());
	}

	pollGateway(): Observable<string> {
		let observableGetState = this.getGateway().map(dev => dev.state); //.do(s => { console.debug('got gateway state ' + s); });
		return observableGetState.delay(1000).repeat();
	}

	setDevState(devId: string, val: string) {
		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });
		return this.http.post(this._url + "devices/" + devId + "/state", JSON.stringify({ "state": val }), options)
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	getCachedDev(id: string): Device {
		return this._mapDevs.get(id);
	}

	private refreshFavoritesSet(favs: string[]) {
		this._setFavorites.clear();
		if (favs != null) {
			favs.forEach(fav => { this._setFavorites.add(fav); });
		}
	}
	getFavorites(bFilter: boolean): string[] {
		var ar: string[];
		ar = <string[]>JSON.parse(localStorage.getItem(ChsService.FAVORITES));
		if (ar == null) ar = [];
		if (bFilter) { // gelöschte Geräte herausfiltern
			let arnew = ar;
			ar = [];
			ar.forEach(fav => {
				if (this._mapDevs.has(fav))
					arnew.push(fav);
			});
			ar = arnew;
		}
		this.refreshFavoritesSet(ar);
		return ar;
	}

	setFavorites(favs: string[]) {
		localStorage.setItem(ChsService.FAVORITES, JSON.stringify(favs));
		this.refreshFavoritesSet(favs);
	}

	setFavorite(id: string, value: boolean) {
		if (!value)
			this.removeFavorite(id);
		else
			this.addFavorite(id);
	}


	addFavorite(fav: string) {
		let favs = this.getFavorites(true);
		if (!(favs.indexOf(fav) >= 0)) {
			favs.push(fav);
			this.setFavorites(favs);
		}
	}

	removeFavorite(fav: string) {
		let favs = this.getFavorites(true);
		var newfavs: string[] = [];
		favs.forEach(elem => {
			if (elem !== fav)
				newfavs.push(elem);
		});
		this.setFavorites(newfavs);
	}

	private storeSettings(settings: SettingsRead) {
		if (settings == null) return;
		for (var key in settings) {
			try {
				this._mapSettings.set(key, settings[key]);
			} catch (e) {
				/*	if (e instanceof SyntaxError) {
						console.debug("Setting " + key + " ist kein JSON:" + e.message);
					}*/
			}
		}
	}

	private applySettingsToDevs() {
		// Favoriten
		this.getFavorites(true);
		this._mapDevs.forEach(dev => {
			dev.favorite = this._setFavorites.has(dev.id);
		});
	}

	getStoredSetting(id: string): any {
		return this._mapSettings.get(id);
	}

	getSettings(): Observable<any> {
		return this.http.get(this._url + "settings")
			.catch((err, caught) => this.handleError(err, caught, this))
			.map(res => { var settings = <SettingsRead>res.json(); return settings; })
			.do(settings => { this.storeSettings(settings); this.onGotSettings.emit(); })
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	getSetting<T>(id: string): Observable<T> {
		return this.http.get(this._url + "settings/" + id)
			.catch((err, caught) => this.handleError(err, caught, this))
			.do(res => {
				var setting: SettingsRead;
				try {
					setting = <SettingsRead>res.json();
					this.storeSettings(setting);
				} catch (ex) { }
			})
			.map(res => {
				var setting = <SettingsRead>res.json();
				if (setting && setting[id] !== undefined) {
					try {
						return <T>setting[id];
					} catch (e) {
						return null;
					}
				}
				return null;
			})
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	pollSetting<T>(id: string): Observable<T> {
		let observableGetState = this.getSetting(id);
		return observableGetState.delay(1000).repeat();
	}

	setSetting(id: string, value: any) {
		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });
		let sett = { key: id, value: value };
		let body = JSON.stringify(sett);
		return this.http.post(this._url + "settings", body, options).do(
			() => { this._mapSettings.set(id, value); })
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	deleteSetting(id: string) {
		return this.http.delete(this._url + "settings/" + id)
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	renameRoom(id: string, newName: string) {
		let room = this.Rooms.get(id);
		if (room) {
			room.name = newName;
			this.writeDevice(room).subscribe(n => { }, err => { }, () => { });
		}
	}

	newRoom(): Observable<Device> {
		var nIdCount = 0;
		var newId: string = "ROOM.";
		var nNameCount = 0;
		var newName: string = "Neuer Raum ";

		var setIds = new Set<string>(this._mapDevs.keys());
		var setNames = new Set<string>();

		this._mapRooms.forEach(dev => {
			setNames.add(dev.name);
		});

		while (setIds.has(newId + nIdCount.toString()))
			++nIdCount;
		while (setNames.has(newName + nNameCount.toString()))
			++nNameCount;

		let dev = { id: newId + nIdCount.toString(), name: newName + nNameCount.toString(), type: "room", sort: maxSort(this._mapRooms) + 1 };

		return this.newDevice(dev, d => {
			this._mapRooms.set(d.id, d);
		});
	}

	public startUpdate(): Observable<any> {
		let body = JSON.stringify({ "update": true });
		let headers = new Headers({ "Content-Type": "application/json" });
		let options = new RequestOptions({ headers: headers });

		return this.http.post(this._url + "update", body, options)
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	public setDateTime(st: SettingTime): Observable<any> {
		let options: RequestOptions = new RequestOptions({ headers: new Headers({ "Content-Type": "application/json" }) });

		return this.http.post(this._url + "time", JSON.stringify(st), options)
			.catch((err, caught) => this.handleError(err, caught, this));
	}

	public getDateTime(): Observable<SettingTime> {
		return this.http.get(this._url + "time")
			.catch((err, caught) => this.handleError(err, caught, this))
			.map(res => { return <SettingTime>res.json(); });
	}

	getTime(): Observable<SettingTime> {
		return this.http.get(this._url + "time").catch((err, caught) => this.handleError(err, caught, this))
			.map(res => <SettingTime>res.json());
	}


	subscribeToStateChange(callback: any): Subscription {
		if (this.stateChangingSubscriptionCount === 0) {
			this.stateChangingSubscription = this.stateChanging.subscribe(changes => {
				this.stateChanged.emit(changes);
			});
		}
		this.stateChangingSubscriptionCount += 1;
		return this.stateChanged.subscribe((changes: IStateChange[]) => { if (changes.length > 0) callback(changes); });
	}

	unsubscribeFromStateChange(sub: Subscription) {
		sub.unsubscribe();
		if (this.stateChangingSubscriptionCount > 0) {
			this.stateChangingSubscriptionCount -= 1;
			if (this.stateChangingSubscriptionCount === 0 && this.stateChangingSubscription) {
				this.stateChangingSubscription.unsubscribe();
				this.stateChangingSubscription = null;
			}
		}
	}

	private handleError(error: Response, caught: Observable<any>, chs: ChsService): Observable<any> {
		console.log(error);
		chs.onError.emit(error);
		return new ErrorObservable(error);
		//return Observable.throw(error);
	}
}