import { Component, ElementRef, EventEmitter, Input, Output, AfterViewInit, OnInit, OnDestroy, ViewChild } from "@angular/core";
import { TranslationService } from "angular-l10n";
import { ChsService, IStateChange } from "../chs.service";
import { Subscription } from "rxjs/Subscription";
import { Device } from "./device";
import { jqxSliderComponent } from "../jqwidgets-ts/angular_jqxslider";

@Component({
	moduleId: module.id,
	selector: "device-editor",
	templateUrl: "deviceeditor.component.html"
})
export class DeviceeditorComponent implements OnInit, OnDestroy, AfterViewInit {
	@Input() public dev: Device;
	@Input() public showDetails: boolean;
	@Input() public showSortHandle: boolean = true;
	@Output() onDeleted = new EventEmitter<string>();

	@ViewChild("nameInput") private _input: ElementRef;
	//@ViewChild("slider") private _slider: jqxSliderComponent;
	private _edit: boolean = false;
	private _oldName: string;
	private _state: string;
	private _id: string;
	private _value: number = -1;
	private _sliderVal: number = -1;
	private _sliding: boolean = false;
	private _valueLabel: string = "---%";
	private _automatic: boolean = false;
	private _changeSubscription: Subscription;

	constructor(public translation: TranslationService,
		private el: ElementRef, private _chsService: ChsService) {
	}

	onClose(): void {
		this.onDeleted.emit(this.dev.id);
	}

	onEditToggle(): void {
		// wird nicht aufgerufen, wenn der onValueChange-Handler aktiv ist
		if (this._edit) {
			this._edit = false;
			if (this._oldName !== this.dev.name) {
				this._chsService.writeDevice(this.dev).subscribe(() => { });
			}
		} else {
			this._edit = true;
			this._oldName = this.dev.name;
			this._input.nativeElement.focus();
		}
	}

	ngOnInit(): void {
		this._id = this.dev.id.replace(".", "__");
		this._state = ChsService.getTypeState(this.dev.type);
		this._valueLabel = "---%";
		if (this.dev.readings) {
			let readings = <Map<string, string | number | boolean>>(this.dev.readings);
			this._value = <number>readings.get("current");
			if (this._value === undefined) {
				this._value = -1;
			} else {
				this._valueLabel = this._value.toString() + "%";
				this._sliderVal=this._value +1;
			}
			this._automatic = <boolean>readings.get("automatic");
		} else {
			this._value = -1;
			this._automatic = false;

		}
	}


	ngAfterViewInit(): void {
		if (this.dev.type !== "scene" && this.dev.type !== "group") {
			this._changeSubscription = this._chsService.subscribeToStateChange((changes: IStateChange[]) => {
				for (let sc of changes) {
					if (sc.devId === this.dev.id) {
						if (sc.newState.state != null) {
							this._value = sc.newState.state;
							this._valueLabel = this._value.toString() + "%";
							if (this._state === "slider" && sc.newState.driving !== undefined && !sc.newState.driving) {
								//this._slider.setValue(this._value);
								this._sliderVal=this._value + 1;
								if (this._sliderVal === 0) { this._sliderVal = 1; } // Umgehung eines Bugs im Slider 0 wird nicht angezeigt
							}
						}
						if (sc.newState.auto != null) {
							this._automatic = sc.newState.auto;
						}
						break;
					}
				}

			});
			if (this._state === "slider" && this._value !== -1) {
				//this._slider.setValue(this._value);
				this._sliderVal=this._value + 1;
			}
		}
	}

	ngOnDestroy(): void {
		this._chsService.unsubscribeFromStateChange(this._changeSubscription);
	}

	onSliderChange(val: any): void {
		if (val.type==="slideStart")  {
			this._sliding=true;
			return;
		}

		if (this._sliding && val.type!=="slideEnd") {
			return;
		}
		this._sliding=false;

		//console.debug("slider event " + val.type + ", args type " + val.args.type + " value " + val.args.value);
		var num: number = Math.round(val.args.value) - 1;
		if (this._value > num + 1 || this._value < num - 1) {
			//console.debug("drive to " + num);
			this.drive("drivePos " + num.toString());
		}
	}

	onSwitchChange(val: any): void {
		if (val.args.checked!=(this._value==100)) {
			this.drive(val.args.checked ? "on" : "off");
		}
	}

	onAutoChange(val: any): void {
		if (val.args.checked != this._automatic) {
			this.drive(val.args.checked ? "autoOn" : "autoOff");
			this._automatic = val.args.checked;
		}
	}

	onFavChange(): void {
		this.dev.favorite = !this.dev.favorite;
		this._chsService.setFavorite(this.dev.id, this.dev.favorite);
	}

	onFocused(event: any): void {
		if (!this._edit) {
			this._oldName = this.dev.name;
			this._edit = true;
		}
	}

	onValueChanged(): void {
		if (this._input.nativeElement.value !== this.dev.name) {
			this.dev.name = this._input.nativeElement.value; 
		}
		if (this._edit) {
			this.onEditToggle();
		}
	}

	private drive(cmd: string): void {
		this._chsService.setDevState(this.dev.id, cmd).subscribe(() => { });
	}

	onDriveUp(): void {
		this.drive("driveUp");
	}

	onDriveDown(): void {
		this.drive("driveDown");
	}

	onDriveStop(): void {
		this.drive("stop");
	}

	onDriveZP1(): void {
		this.drive("drivePos1");
	}

	onDriveZP2(): void {
		this.drive("drivePos2");
	}

	onLearnZP1(): void {
		this.drive("savePos1");
	}

	onLearnZP2(): void {
		this.drive("savePos2");
	}

}