import { Component, AfterViewInit, ElementRef } from "@angular/core";
import { Router } from "@angular/router";
import { ChsService } from "../chs.service";
import { Device } from "./device";
import { sortRealDevs } from "../utils";

@Component({
	template: `
			<div class='panel panel-primary'>
				<div class='panel-heading' translate>DEVS</div>
				<jqxSortable id='devicelist' class='panel-body' [handle]='".handle"' (onStop)="orderChanged()">
					<div *ngFor="let device of _devices" class='panel panel-default' id="{{device.id}}">
						<device-editor [dev]="device" (onDeleted)="onDeviceDelete($event)"></device-editor>
					</div>
				</jqxSortable>
			</div>
			<p></p>
			<button type="button" class="btn btn-default" (click)="onSearchDevices()" [disabled]="_stickNA" translate>STARTTEACHIN</button>
			<p></p>
			`
})
export class DevicesComponent implements AfterViewInit {
	constructor(private el: ElementRef, private _chsService: ChsService, private _router: Router) {
	}

	private _devices: Device[] = [];
	private _stickNA: boolean = true;


	private initDevices() {
		this._chsService.getDevices().subscribe(x => { }, err => { }, () => {
			try {
				this._stickNA = (this._chsService.Gateway.state==="na");
				this._devices = sortRealDevs(this._chsService.RealDevs.values());
			} catch (ex) {}
		});
	}

	ngAfterViewInit() {
		this.initDevices();
	}

	onSearchDevices() {
		this._router.navigate(["cfg/teachin"]);
	}

	onDeviceDelete(id: string) {
		this._chsService.deleteDevice(id).subscribe(() => {
			this._devices = this._devices.filter(d => d.id != id);
		});
	}

	private orderChanged() {
		let items = jQuery(this.el.nativeElement).find(".panel.panel-default");
		var ar: Device[] = [];

		for (var index = 0; index < items.length; index++) {
			var element = jQuery(items[index]);
			var dev = this._chsService.RealDevs.get(element.attr("id"));
			dev.sort_dev = index;
			ar.push(dev);
		}

		this._chsService.writeDevices(ar).subscribe(next => { }, err => { }, () => { });
	}
}