"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var angular_l10n_1 = require("angular-l10n");
var chs_service_1 = require("../chs.service");
var DeviceDriveComponent = (function () {
    function DeviceDriveComponent(translation, el, _chsService) {
        this.translation = translation;
        this.el = el;
        this._chsService = _chsService;
        this.onDeleted = new core_1.EventEmitter();
        this._value = -1;
        this._valueLabel = "---%";
        this._sliderVal = -1;
        this._sliding = false;
    }
    DeviceDriveComponent.prototype.onClose = function () {
        this.onDeleted.emit(this.dev.id);
    };
    DeviceDriveComponent.prototype.ngOnInit = function () {
        this._id = this.dev.id.replace(".", "__");
        this._state = this._chsService.getDeviceState(this.dev.id);
        this._valueLabel = "---%";
        if (this.dev.readings) {
            var readings = this.dev.readings;
            this._value = readings.get("current");
            if (this._value === undefined) {
                this._value = -1;
            }
            else {
                this._valueLabel = this._value.toString() + "%";
                this._sliderVal = this._value + 1;
            }
            this._automatic = readings.get("automatic");
        }
        else {
            this._value = -1;
            this._automatic = false;
        }
    };
    DeviceDriveComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (this.dev.type !== "scene" && this.dev.type !== "group") {
            this._changeSubscription = this._chsService.subscribeToStateChange(function (changes) {
                for (var _i = 0, changes_1 = changes; _i < changes_1.length; _i++) {
                    var sc = changes_1[_i];
                    if (sc.devId === _this.dev.id) {
                        if (sc.newState.state != null) {
                            _this._value = sc.newState.state;
                            _this._valueLabel = _this._value.toString() + "%";
                            if (_this._state === "slider" && sc.newState.driving !== undefined && !sc.newState.driving) {
                                //this._slider.setValue(this._value);
                                _this._sliderVal = _this._value + 1;
                                if (_this._sliderVal === 0) {
                                    _this._sliderVal = 1;
                                } // Umgehung eines Bugs im Slider 0 wird nicht angezeigt
                            }
                        }
                        if (sc.newState.auto != null) {
                            _this._automatic = sc.newState.auto;
                        }
                        break;
                    }
                }
            });
            if (this._state === "slider" && this._value !== -1) {
                this._sliderVal = this._value + 1;
                //this._slider.setValue(this._value);
            }
        }
    };
    DeviceDriveComponent.prototype.ngOnDestroy = function () {
        if (this._changeSubscription) {
            this._chsService.unsubscribeFromStateChange(this._changeSubscription);
        }
    };
    DeviceDriveComponent.prototype.onSliderChange = function (val) {
        if (val.type === "slideStart") {
            this._sliding = true;
            return;
        }
        if (this._sliding && val.type !== "slideEnd") {
            return;
        }
        this._sliding = false;
        //console.debug("slider event " + val.type + ", args type " + val.args.type + " value " + val.args.value);
        var num = Math.round(val.args.value) - 1;
        if (this._value > num + 1 || this._value < num - 1) {
            //console.debug("drive to " + num);
            this.drive("drivePos " + num.toString());
        }
    };
    DeviceDriveComponent.prototype.onSwitchChange = function (val) {
        if (val.args.checked != (this._value == 100)) {
            this.drive(val.args.checked ? "on" : "off");
        }
    };
    DeviceDriveComponent.prototype.onAutoChange = function (val) {
        if (val.args.checked != this._automatic) {
            this.drive(val.args.checked ? "autoOn" : "autoOff");
            this._automatic = val.args.checked;
        }
    };
    DeviceDriveComponent.prototype.drive = function (cmd) {
        this._chsService.setDevState(this.dev.id, cmd).subscribe(function () { });
    };
    DeviceDriveComponent.prototype.onDriveUp = function () {
        this.drive("driveUp");
    };
    DeviceDriveComponent.prototype.onDriveDown = function () {
        this.drive("driveDown");
    };
    DeviceDriveComponent.prototype.onDriveStop = function () {
        this.drive("stop");
    };
    DeviceDriveComponent.prototype.onDriveZP1 = function () {
        this.drive("drivePos1");
    };
    DeviceDriveComponent.prototype.onDriveZP2 = function () {
        this.drive("drivePos2");
    };
    DeviceDriveComponent.prototype.onStartScene = function () {
        this.drive("on");
    };
    DeviceDriveComponent.prototype.onStopScene = function () {
        this.drive("stop");
    };
    return DeviceDriveComponent;
}());
__decorate([
    core_1.Input(),
    __metadata("design:type", Object)
], DeviceDriveComponent.prototype, "dev", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", Boolean)
], DeviceDriveComponent.prototype, "showDetails", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", Boolean)
], DeviceDriveComponent.prototype, "showClose", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", Boolean)
], DeviceDriveComponent.prototype, "sortHandle", void 0);
__decorate([
    core_1.Output(),
    __metadata("design:type", Object)
], DeviceDriveComponent.prototype, "onDeleted", void 0);
DeviceDriveComponent = __decorate([
    core_1.Component({
        moduleId: module.id,
        selector: "device-drive",
        templateUrl: "drive.component.html"
    }),
    __metadata("design:paramtypes", [angular_l10n_1.TranslationService,
        core_1.ElementRef, chs_service_1.ChsService])
], DeviceDriveComponent);
exports.DeviceDriveComponent = DeviceDriveComponent;
//# sourceMappingURL=drive.component.js.map