import { Component, ElementRef, EventEmitter, Input, Output, AfterViewInit, OnInit, OnDestroy, ViewChild } from "@angular/core";
import { Translation, LocaleService, TranslationService } from "angular-l10n";
import { ChsService, IStateChange } from "../chs.service";
import { Subscription } from "rxjs/Subscription";
import { Device } from "./device";
import { jqxSliderComponent } from "../jqwidgets-ts/angular_jqxslider";

@Component({
	moduleId: module.id,
	selector: "device-drive",
	templateUrl: "drive.component.html"
})
export class DeviceDriveComponent implements OnInit, OnDestroy, AfterViewInit {
	@Input() public dev: Device;
	@Input() public showDetails: boolean;
	@Input() public showClose: boolean;
	@Input() public sortHandle: boolean;
	@Output() onDeleted = new EventEmitter<string>();

	//@ViewChild("slider") private _slider: jqxSliderComponent;
	private _state: string;
	private _id: string;
	private _buttonChecked: boolean;
	private _value: number = -1;
	private _valueLabel: string = "---%";
	private _sliderVal: number = -1;
	private _sliding: boolean = false;


	private _automatic: boolean;
	private _changeSubscription: Subscription;

	constructor(public translation: TranslationService,
		private el: ElementRef, private _chsService: ChsService) {
	}

	onClose(): void {
		this.onDeleted.emit(this.dev.id);
	}

	ngOnInit(): void {
		this._id = this.dev.id.replace(".", "__");
		this._state = this._chsService.getDeviceState(this.dev.id);
		this._valueLabel = "---%";
		if (this.dev.readings) {
			let readings = (<Map<string, string | number | boolean>>this.dev.readings);
			this._value = <number>readings.get("current");
			if (this._value === undefined) {
				this._value = -1;
			} else {
				this._valueLabel = this._value.toString() + "%";
				this._sliderVal = this._value + 1;
			}
			this._automatic = <boolean>readings.get("automatic");
		} else {
			this._value = -1;
			this._automatic = false;
		}
	}

	ngAfterViewInit(): void {
		if (this.dev.type !== "scene" && this.dev.type !== "group") {
			this._changeSubscription = this._chsService.subscribeToStateChange((changes: IStateChange[]) => {
				for (let sc of changes) {
					if (sc.devId === this.dev.id) {
						if (sc.newState.state != null) {
							this._value = sc.newState.state;
							this._valueLabel = this._value.toString() + "%";
							if (this._state === "slider" && sc.newState.driving !== undefined && !sc.newState.driving) {
								//this._slider.setValue(this._value);
								this._sliderVal = this._value + 1;
								if (this._sliderVal === 0) { this._sliderVal = 1; } // Umgehung eines Bugs im Slider 0 wird nicht angezeigt
							}
						}
						if (sc.newState.auto != null) {
							this._automatic = sc.newState.auto;
						}
						break;
					}
				}

			});
			if (this._state === "slider" && this._value !== -1) {
				this._sliderVal = this._value + 1;
				//this._slider.setValue(this._value);
			}
		}
	}

	ngOnDestroy(): void {
		if (this._changeSubscription) {
			this._chsService.unsubscribeFromStateChange(this._changeSubscription);
		}
	}

	onSliderChange(val: any): void {
		if (val.type === "slideStart") {
			this._sliding = true;
			return;
		}

		if (this._sliding && val.type !== "slideEnd") {
			return;
		}
		this._sliding = false;

		//console.debug("slider event " + val.type + ", args type " + val.args.type + " value " + val.args.value);
		var num: number = Math.round(val.args.value) - 1;
		if (this._value > num + 1 || this._value < num - 1) {
			//console.debug("drive to " + num);
			this.drive("drivePos " + num.toString());
		}
	}

	onSwitchChange(val: any): void {
		if (val.args.checked != (this._value == 100)) {
			this.drive(val.args.checked ? "on" : "off");
		}
	}

	onAutoChange(val: any): void {
		if (val.args.checked != this._automatic) {
			this.drive(val.args.checked ? "autoOn" : "autoOff");
			this._automatic = val.args.checked;
		}
	}

	private drive(cmd: string): void {
		this._chsService.setDevState(this.dev.id, cmd).subscribe(() => { });
	}

	onDriveUp(): void {
		this.drive("driveUp");
	}

	onDriveDown(): void {
		this.drive("driveDown");
	}

	onDriveStop(): void {
		this.drive("stop");
	}

	onDriveZP1(): void {
		this.drive("drivePos1");
	}

	onDriveZP2(): void {
		this.drive("drivePos2");
	}

	onStartScene(): void {
		this.drive("on");
	}

	onStopScene(): void {
		this.drive("stop");
	}




}