import { Component, AfterViewInit, OnDestroy, ElementRef, Inject, ViewEncapsulation } from "@angular/core";
import { Router } from "@angular/router";
import {Observable} from "rxjs/Observable";
import {Subscription} from "rxjs/Subscription";
import {ChsService} from "../chs.service";
import {Device} from "./device";
import {sortRealDevs, maxSortRealDev} from "../utils";
import {DeviceeditorComponent} from "./deviceeditor.component";

@Component({
	moduleId: module.id,
	selector: "commeo-devices",
	templateUrl: "teachin.component.html"
})
export class TeachInComponent implements AfterViewInit, OnDestroy {
	constructor(private el: ElementRef, private _chsService: ChsService, private _router: Router) {
		this.bPairing=true;
	}

	//private _devicelist: JQuery;
	private _devices: Device[] = [];
	private _setExistinDevIds = new Set<string>();
	private static SETTING="teachinOldDevs";
	private static MAXSORT="teachinMaxSort";
	private bPairing: boolean;
	private _subScanning: Subscription;
	
	private initDevices() {
		//this._chsService.setSetting(ChsService.DEVORDER, [ "CGW.0", "CGW.1"]).subscribe(() => { console.debug("Setting geschrieben")});
		
		
		this._chsService.getDevices().subscribe((x: any)=> {}, (err: any) => {}, () => {
			try {
				let bScanning:boolean = (this._chsService.Gateway.state==="scan");
				if (!bScanning) {
					let devs=sortRealDevs(this._chsService.RealDevs.values());
					if (devs.length>0) {
						let nMax: number = devs[devs.length-1].sort_dev;
						if (nMax===undefined) { nMax=0; }
						if (nMax<devs.length-1) { nMax=devs.length; }
						localStorage.setItem(TeachInComponent.MAXSORT, nMax.toString());
						this._setExistinDevIds.clear();
						devs.forEach(dev => {
							this._setExistinDevIds.add(dev.id);
						});
						localStorage.setItem(TeachInComponent.SETTING, JSON.stringify(this._setExistinDevIds));

					}

					let subScr: Subscription = this._chsService.beginTeachIn().subscribe(x => {
							console.debug("next " + x.toString()); if (x==="ready") { this.fillNewDevices(); subScr.unsubscribe(); 
						}},	err => { console.debug("err"); }, () => { console.debug("complete");
					});
				}
				else { // Scanning
					let existingIds=localStorage.getItem(TeachInComponent.SETTING);
					if (existingIds && existingIds.length>0)
						this._setExistinDevIds=<Set<string>>JSON.parse(existingIds);

					this._subScanning = this._chsService.pollGateway().subscribe(x => { 
							console.debug("next " + x.toString()); if (x==="ready") { this.fillNewDevices(); this._subScanning.unsubscribe(); 
						}},	err => { console.debug("err"); }, () => { console.debug("complete");  
					});
				}
			} catch (ex) {

			}
		});
	}

	private fillNewDevices() {
		this._chsService.getDevices().subscribe(x=> {}, err => {}, () => {
			this.bPairing=false;
			let nSortDev=0;
			let strMax=localStorage.getItem(TeachInComponent.MAXSORT);
			if (strMax) {
				nSortDev=Number.parseInt(strMax);
			}
			this._devices=[];
			this._chsService.RealDevs.forEach(dev => {
				if (!this._setExistinDevIds.has(dev.id)) {
					dev.sort_dev=(++nSortDev);
					this._devices.push(dev);
				}
			});
			/* Keine Geräte schreiben. Sie sind schom im Stick.
			if (this._devices.length>0) {
				this._chsService.writeDevices(this._devices).subscribe();
			}*/
		});
	}

	ngAfterViewInit() {
		this.initDevices();
	}

	ngOnDestroy() {
		if (this._subScanning) {
			this._subScanning.unsubscribe();
		}
	}

	onBack() {
		this._router.navigate(["cfg/devices"]);
	}

	onDeviceDelete(id: string) {
		this._chsService.deleteDevice(id).subscribe( () => {
			this._devices = this._devices.filter(d => d.id!==id );
		});
	}
}