import { Component, OnInit, OnChanges, SimpleChange, Input } from "@angular/core";
import {ChsService} from "../chs.service";
import {Device} from "../devices/device";
import { sortDevs } from "../utils";

@Component({
	selector: "room-drive",
	template: `
		<div class='panel panel-primary'>
			<div class='panel-heading'>{{room.name}}</div>
			<div class='panel-body container-fluid'>
				<div *ngFor="let device of _devs; let idx = index; let lastDev = last"  class="col-md-3">
					<device-drive [dev]="device"></device-drive>	
				</div>
			</div>
		</div>
	`
})
export class OneRoomDriveComponent implements OnChanges {
	@Input() room: Device;
	private _devs: Device[] = [];

	
	constructor(private _chsService: ChsService) { 
	}

	private initData(): void {
		if (!this.room)  { return; }
		try {
			var ar:Device[]=[];
			this._chsService.RealDevs.forEach( dev => {
				//console.debug("raum " + this.room.id + " gerät." + dev.id + " in raum " + dev.room);
				if (dev.room === this.room.id) {
					ar.push(dev);
				}
			});
			this._devs=sortDevs(ar.values());
		} catch (ex) {

		}
	}

	ngOnChanges(changes: { [propertyName: string]: SimpleChange }): void {
		for (let propName in changes) {
			if (propName === "room") {
				/*let prop: SimpleChange = changes[propName];
				let roomNew: any = <Device>prop.currentValue;
				let roomOld: any = <Device>prop.previousValue;
				*/
				this.initData();
			}
		}
	}
}

@Component({
	template: `
		<div *ngFor="let room of _rooms">
			<room-drive [room]='room'></room-drive>
		</div>
		<div class='panel panel-primary'>
			<div class='panel-heading' translate>NOROOMDEVS</div>
			<div class='panel-body container-fluid'>
				<div *ngFor="let device of _noroomDevs; let idx = index; let lastDev = last"  class="col-md-3">
					<device-drive [dev]="device"></device-drive>	
				</div>
			</div>
		</div>			
	`
})
export class RoomsDriveComponent implements OnInit {
	private _rooms: Device[];
	private _noroomDevs: Device[] = [];

	constructor(private _chsService: ChsService) {
	}

	ngOnInit(): void {
		this._noroomDevs=[];
		this._chsService.getDevices().subscribe(x=> {}, err => {}, () => {
			try {
				this._rooms=sortDevs(this._chsService.Rooms.values());
				let sortedDevs: Device[] = sortDevs(this._chsService.RealDevs.values());
				sortedDevs.forEach(dev => {
					if (!dev.room || dev.room.length===0) {
						this._noroomDevs.push(dev);
					}
				});

			} catch (ex) {
			}
		});
	}
}