import { Component, ElementRef, EventEmitter, ViewChild, Input, Output, OnInit } from "@angular/core";
import { Router } from "@angular/router";
import { TranslationService } from "angular-l10n";
import { ChsService } from "../chs.service";
import { Device } from "../devices/device";

@Component({
	moduleId: module.id,
	selector: "group-editor",
	templateUrl: "groupeditor.component.html"
})
export class GroupeditorComponent implements OnInit {
	@Input() public dev: Device;
	@Output() onDeleted = new EventEmitter<string>();

	@ViewChild("nameInput") private _input: ElementRef;
	private _edit: boolean = false;
	private _oldName: string;
	private _state: string;
	private _id: string;
	private _buttonChecked: boolean;
	private _sliderValue: number;
	private _sliding: boolean = false;
	private _timers: Device[] = [];

	constructor(public translation: TranslationService, private _chsService: ChsService,
		private _router: Router) {
	}

	onClose() {
		this.onDeleted.emit(this.dev.id);
	}

	onEditToggle() {
		// wird nicht aufgerufen, wenn der onValueChange-Handler aktiv ist
		if (this._edit) {
			this._edit = false;
			if (this._oldName !== this.dev.name) {
				this._chsService.writeDevice(this.dev).subscribe(() => { });
			}
		}
		else {
			this._edit = true;
			this._oldName = this.dev.name;
			this._input.nativeElement.focus();
		}
	}

	onGotoMembers() {
		this._router.navigate(["cfg/groupmembers", this._id]);
	}

	onTimers() {
		this._router.navigate(["cfg/timers", this._id]);
	}

	onSliderChange(val: any) {
		if (val.type==="slideStart")  {
			this._sliding=true;
			return;
		}

		if (this._sliding && val.type!=="slideEnd") {
			return;
		}
		this._sliding=false;

		//console.debug("slider event " + val.type + ", args type " + val.args.type + " value " + val.args.value);
		var num: number = Math.round(val.args.value) - 1;
		this.drive("drivePos " + num.toString());
	}

	onSwitchChange(val: any) {
		this.drive(val.args.checked ? "on" : "off");
	}

	ngOnInit() {
		this._id = this.dev.id.replace(".", "__");
		this._state = this._chsService.getGroupState(this.dev.id);
		this._timers=[];
		this._chsService.Timers.forEach(dev => {  if(dev.type==="timer" && dev.device===this.dev.id) { this._timers.push(dev); } } );
	}

	onFocused(event: any) {
		if (!this._edit) {
			this._oldName = this.dev.name;
			this._edit = true;
		}
	}

	onValueChanged() {
		if (this._input.nativeElement.value !== this.dev.name) {
			this.dev.name = this._input.nativeElement.value;
		}
		if (this._edit) {
			this.onEditToggle();
		}
	}

	onFavChange() {
		this.dev.favorite=!this.dev.favorite;
		this._chsService.setFavorite(this.dev.id, this.dev.favorite);
	}

	private drive(cmd: string) {
		this._chsService.setDevState(this.dev.id, cmd).subscribe(() => { });
	}

	onDriveUp() {
		this.drive("driveUp");
	}

	onDriveDown() {
		this.drive("driveDown");
	}

	onDriveStop() {
		this.drive("stop");
	}

	onDriveZP1() {
		this.drive("drivePos1");
	}

	onDriveZP2() {
		this.drive("drivePos2");
	}

}