import { Component, OnInit, EventEmitter } from "@angular/core";
import { Router, ActivatedRoute } from "@angular/router";
import { Translation, TranslationService } from "angular-l10n";
import { ChsService } from "../chs.service";
import { TranslateService } from "../translation.service";
import { GroupDataAdapter } from "./groupDataAdapter";
import { Device } from "../devices/device";

@Component({
	moduleId: module.id,
	selector: "commeo-groups",
	template: `
		<div class='panel panel-primary'>
			<div class='panel-heading'><span translate>ASSIGNMEMBERS</span> {{_group?.name}}</div>
			<div class='panel-body'>
				<div *ngIf='_group && _visible'>
					<jqxKanban [width]="'100%'" [itemRenderer]='_itemRenderer'
						[source]='_source' [template]='_template' [resources]='_resources'
						[columns]='_columns' (onItemMoved)='onItemMoved($event)'>
					</jqxKanban>
				</div>
			</div>
		</div>
		<p></p>
		<button type="button" class="btn btn-default" (click)="onBack()" translate>BACK</button>
		<p></p>
	`
})
export class GroupmembersComponent   implements OnInit {
	private _id: string;
	private _group: Device;
	private _visible: boolean = true;
	private _evtVisible: EventEmitter<boolean>= new EventEmitter<boolean>(true);
	private _da: GroupDataAdapter;
	private _columns = [{ text: "NOGROUP", dataField: "nogroup" }, { text: "", dataField: "" }];
	private _source: any;
	private _resources: any;
	private _template = `
					<div groups class='jqx-kanban-item' id=''>
						<div groups class='jqx-kanban-item-color-status'></div>
						<div groups style='display: none;' class='jqx-kanban-item-avatar'></div>
						<div groups class='jqx-kanban-item-text'></div>
						<div groups style='display: none' class='jqx-kanban-item-footer'></div>
					</div>
				`;
	private _itemRenderer = function (element:any, item: any, resource:any): void {
		jQuery(element).find(".jqx-kanban-item-color-status").html(
			"<img class='menu-icon' src='"+resource.image +"' height='16px' /><span style='line-height: 23px; margin-left: 5px; color:white;'>" + resource.name + "</span>");
	};

	constructor(public translation: TranslationService,
		private _route: ActivatedRoute, private _router: Router,
		private _chsService: ChsService, private _transService: TranslateService) {
		
		this._id = this._route.snapshot.params["id"].replace("__", ".");
		this._evtVisible.subscribe( (v: boolean) => { this.visibleTrick(v);  } );
	}

	private visibleTrick(v: boolean) {
		let g: Device = this._chsService.getCachedDev(this._id);
		this.initData(g);
		this._visible=v;
	}

	private initData(s: Device) {
		if (s==null) {
			return;
		}
		this._group = s;
		let strNoGroup: string = this.translation.translate("NOGROUP");
		this._columns=[{ text: strNoGroup, dataField: "nogroup" }, { text: this._group.name, dataField: this._group.id }];
		this._da = new GroupDataAdapter(this._chsService);
		this._resources = this._da.resources();
		this._da.setDevicesForGroup(this._group.id);
		this._source = this._da.dataAdapter();
	}

	ngOnInit() {
		let s = this._chsService.getCachedDev(this._id);
		if (s) {
			this.initData(s);
		} else {
			this._chsService.getDevices().subscribe(devs => {
				s = this._chsService.getCachedDev(this._id);
				this.initData(s);
			},
				error => { },
				() => { }
			);
		} 
	}

	private onItemMoved(event: any): void {
		var args = event.args;
		if (args.oldColumn.dataField !== args.newColumn.dataField) {
			let itemData: any = args.itemData;
			let records: any = this._source.records;
			// Originale Id holen aus DataAdapter
			var origid: string = "";
			var recordIndex: number = -1;
			records.forEach((item: any, index: number) => {
				if (item.id === itemData.id) {
					origid = item.origid;
					recordIndex = index;
				}
			});

			if (args.newColumn.dataField === "nogroup") {
				this._chsService.removeFromGroup(origid, args.oldColumn.dataField).subscribe(() => { });
			}	else {
				// prüfen, ob das Gerät zu den anderen Geräte der Gruppe passt
				let groupstate: string=this._chsService.getGroupState(this._group.id);
				let devstate: string=this._chsService.getDeviceState(origid);
				if (groupstate.length===0 || groupstate===devstate) {
					this._chsService.addToGroup(origid, args.newColumn.dataField).subscribe(() => { });
				}	else {
					this._visible=false;
					this._evtVisible.emit(true);
					/*let opts: any = this._kanban.manageAttributes();
					this._kanban.destroy();
					jQuery(this._kanban.elementRef.nativeElement).html("<ng-content></ng-content>");
					this.initData(this._group);
					opts.source=this._source;
					opts.resources=this._resources;
					opts.columns=this._columns;
					this._kanban.createComponent();*/
				}
			}
		}
	}

	onBack(): void {
		this._router.navigate(["cfg/groups"]);
	}

}