import { Component, AfterViewInit, ElementRef } from "@angular/core";
import { ChsService } from "../chs.service";
import { Device } from "../devices/device";
import { GroupDataAdapter } from "./groupDataAdapter";
import { sortDevs } from "../utils";

declare var jQuery: JQueryStatic;

@Component({
	selector: "commeo-groups",
	template: `
			<div class='panel panel-primary'>
				<div class='panel-heading' translate>GROUPS</div>
				<jqxSortable class='panel-body' [handle]="'.handle'" (onStop)='orderChanged()'>
					<div *ngFor="let group of _groups">
						<group-editor [dev]="group" (onDeleted)="onGroupDelete($event)"></group-editor>
					</div>
				</jqxSortable>
			</div>
			<p></p>
			<button type="button" class="btn btn-default" (click)="onNewGroup()" translate>NEWGROUP</button>
			<p></p>
			`
})
export class GroupsComponent implements AfterViewInit {
	constructor(private el: ElementRef, private _chsService: ChsService) {
		this._da = new GroupDataAdapter(_chsService);
	}

	public _da: GroupDataAdapter;

	//private _groups: Device[] = [{ id: 'group.0', name: 'Gruppe 1', type: 'group' }, { id: 'group.1', name: 'Gruppe 2', type: 'group' }];
	private _groups: Device[] = [];

	ngAfterViewInit() {
		this._chsService.getDevices().subscribe(devs => {
			this._da.setDevices();
			try {
				this._groups = sortDevs(this._chsService.Groups.values());
			} catch (ex) {}
		},
			error => { });
	}

	onNewGroup() {
		this._chsService.newGroup().subscribe(group => {
			this._groups.push(group);
		});
	}

	onGroupDelete(id: string) {
		this._chsService.deleteGroup(id).subscribe(() => {
			this._groups = this._groups.filter(g => g.id != id);
		});
	}

	private orderChanged(): void {
		let items = jQuery(this.el.nativeElement).find(".panel.panel-default[id]");
		var ar: Device[] = [];

		for (var index: number = 0; index < items.length; index++) {
			var element: JQuery = jQuery(items[index]);
			var dev: Device = this._chsService.Groups.get(element.attr("id"));
			dev.sort = index;
			ar.push(dev);
		}

		this._chsService.writeDevices(ar).subscribe(next => { }, err => { }, () => { });
	}

}