import { Component, OnInit, OnDestroy, ViewChild, EventEmitter } from "@angular/core";
import { Router, ActivatedRoute } from "@angular/router";
import { Translation, TranslationService } from "angular-l10n";
import { ChsService } from "../chs.service";
import { TranslateService } from "../translation.service";
import { Device } from "../devices/device";
import { jqxKanbanComponent } from "../jqwidgets-ts/angular_jqxkanban";
import { resources, sortDevs, getSortedColumnItems } from "../utils";
import { backcolor } from "../utils";
import { Subscription } from "rxjs/Subscription";

@Component({
	selector: "commeo-roomassign",
	template: `
		<div class='panel panel-primary'>
			<div class='panel-heading' translate>ASSIGNDEVS</div>
			<div class='panel-body'>
				<div *ngIf='_visible'>
					<jqxKanban  [width]="'100%'" 
						[itemRenderer]='_itemRenderer'
						[source]='_da' 
						[template]='_template' 
						[resources]='_resources'
						[columns]='_columns' 
						(onItemMoved)='onItemMoved($event)'>
					</jqxKanban>
				</div>
			</div>
		</div>
		<a role="button" class="btn btn-default" [routerLink]="['/cfg/rooms']" translate>BACK</a>
	`
})
export class RoomAssignComponent   implements OnInit, OnDestroy {
	@ViewChild(jqxKanbanComponent) private _kanban: jqxKanbanComponent;
	private _visible: boolean = false;
	private _columnsStart = [{ text: "ohne Raum", dataField: "noroom" }];
	private _columns = this._columnsStart;
	private _da: any;
	private _resources: any = resources;
	private _onMoved = new EventEmitter(true);
	private _subscriptionMoved: Subscription = null;

	private _template = `
					<div groups class='jqx-kanban-item' id=''>
						<div groups class='jqx-kanban-item-color-status'></div>
						<div groups style='display: none;' class='jqx-kanban-item-avatar'></div>
						<div groups class='jqx-kanban-item-text'></div>
						<div groups style='display: none' class='jqx-kanban-item-footer'></div>
					</div>
				`;

	private _itemRenderer(element: any, item: any, resource: any) {
		jQuery(element).find(".jqx-kanban-item-color-status").html(
			"<img class='menu-icon' src='" + resource.image + "' height='16px' /><span style='line-height: 23px; margin-left: 5px; color:white;'>" + resource.name + "</span>");
	};

	private _source =
	{
		localData: new Array<any>(),
		dataType: "array",
		dataFields: [
			{ name: "id", type: "string" },
			{ name: "status", map: "room", type: "string" },
			{ name: "text", map: "name", type: "string" },
			{ name: "tags", map: "type", type: "string" },
			{ name: "origid", type: "string" },
			{ name: "resourceId", map: "type", type: "string" },
			{ name: "color", type: "string" }
		]
	};

	constructor(public translation: TranslationService,
		private _route: ActivatedRoute, private _router: Router,
		private _chsService: ChsService, private _transService: TranslateService) {
		 
	}

	private _headerIcon = "jqx-icon-close";
	private _canCollapse = false;

	private initData() {
		let rooms:Device[] = sortDevs(this._chsService.Rooms.values());
		this._columns = [this._columnsStart[0]];
		rooms.forEach(room => {
			this._columns.push({ text: room.name, dataField: room.id });
		});
		this._source.localData = [];
		let reaDevs:Device[]=sortDevs(this._chsService.RealDevs.values());
		reaDevs.forEach(dev => {
			this._source.localData.push({ id: dev.id.replace(".", "_"), origid: dev.id, name: dev.name, room: !dev.room ? "noroom" : dev.room, type: dev.type, color: backcolor });
		});

		this._da = new $.jqx.dataAdapter(this._source);
	}

	private getRecord(id: string): any {
		if (!this._da) return null;
		let records: any[] = this._da.records;
		for (let i: number = 0; i < records.length; ++i) {
			if (records[i].id === id) {
				return records[i];
			}
		}
		return null;
	}

	ngOnInit() {
		this._columnsStart[0].text=this.translation.translate("NOROOM");
		this._chsService.getDevices().subscribe(next => { }, err => { }, () => {
			this.initData();
			this._visible=true;
		});
		this._subscriptionMoved =	this._onMoved.subscribe((args: any) => { this.onMovedHandler(args);  } );
	}

	ngOnDestroy() {
		if (this._subscriptionMoved) {
			this._subscriptionMoved.unsubscribe();
			this._subscriptionMoved = null;
		}
	}

	private saveColumnDevs(col: string) {
		let room = (col === "noroom" ? null : col);
		var devsToWrite: Device[] = [];

		let colItems = getSortedColumnItems(this._kanban, col);
		let order = 0;
		colItems.forEach((ci: any ) => {
			let record = this.getRecord(ci.id);
			var colDev: Device = null;
			if (record) {
				colDev = this._chsService.getCachedDev(record.origid);
			}
			if (colDev) {
				colDev.room = room;
				colDev.sort = order;
				++order;
				devsToWrite.push(colDev);
			}
		});
		if (devsToWrite.length > 0) {
			this._chsService.writeDevices(devsToWrite).subscribe(next => { }, err => { }, () => { });
		}
	}

	private onMovedHandler(args: any) {
	// neue Spalte komplett speichern
		this.saveColumnDevs(args.newColumn.dataField);

		if (args.newColumn.dataField !== args.oldColumn.dataField) {
			this.saveColumnDevs(args.oldColumn.dataField);
		}
	}

	private onItemMoved(event: any) {
		var args = event.args;
		this._onMoved.emit(args);

		/*
				let itemData = args.itemData;
				let records = this._da.records;
				// Originale Id holen aus DataAdapter
				var origid: string = "";
				var recordIndex = -1;
				records.forEach((item, index) => {
					if (item.id == itemData.id) {
						origid = item.origid;
						recordIndex = index;
					}
				});
		
				let dev = this._chsService.getCachedDev(origid);
				if (dev) {
					dev.room = newroom;
					this._chsService.writeDevice(dev).subscribe();
				}*/
	}
}