import { Component, ElementRef, EventEmitter, ViewChild, Input, Output, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import {ChsService} from '../chs.service';
import {Device} from '../devices/device';
import {AppComponent} from '../app.component';
import {RoomsComponent} from './rooms.component';
import { Observable } from 'rxjs/Rx';

@Component({
	selector: 'room-editor',
	template: `
			<div  id="{{room.id}}" class='panel panel-default commeo-icon-container' style='margin-bottom: 8px;'><div class='panel-body'>
				<div class='commeo-icon-right1 jqx-icon jqx-icon-close' (click)='onClose()'></div>
				<div class='handle commeo-icon-left1 jqx-icon jqx-icon-sort-handle'></div>
				<p></p>
				<div class="input-group">
  				<input type="text" class="form-control" aria-label="..." [(value)]='room.name' (focus)='onFocused($event)' (change)='onValueChanged()' #nameInput>
  				<div class="input-group-btn">
    				<button type="button" class="btn btn-default" (click)='onEditToggle()'><div class='glyphicon' [class.glyphicon-pencil]='!_edit' [class.glyphicon-ok]='_edit'></div></button>
  				</div>
				</div>
			</div></div>
	`
})
export class RoomeditorComponent implements OnInit {
	@Input() public room: Device;
	@Output() onDeleted = new EventEmitter<string>();

	@ViewChild('nameInput') private _input: ElementRef;
	private _edit: boolean = false;
	private _oldName: string;
	private _id: string;

	constructor(private _chsService: ChsService,
		private _router: Router) {
	}

	onClose() {
		this.onDeleted.emit(this.room.id);
	}

	onEditToggle() {
		// wird nicht aufgerufen, wenn der onValueChange-Handler aktiv ist
		if (this._edit) {
			this._edit = false;
			if (this._oldName != this.room.name) {
				this._chsService.writeDevice(this.room).subscribe(() => { });
			}
		}
		else {
			this._edit = true;
			this._oldName = this.room.name;
			this._input.nativeElement.focus();
		}
	}

	ngOnInit() {
	}

	onFocused(event: any) {
		if (!this._edit) {
			this._oldName = this.room.name;
			this._edit = true;
		}
	}

	onValueChanged() {
		if (this._input.nativeElement.value != this.room.name)
			this.room.name = this._input.nativeElement.value;
		if (this._edit)
			this.onEditToggle();
	}

}