import { Component, ElementRef, OnInit } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { ChsService } from "../chs.service";
import { Device } from "../devices/device";
import { sortDevs } from "../utils";

@Component({
	selector: "commeo-rooms",
	template: `
		<div class='panel panel-primary'>
			<div class='panel-heading' translate>ROOMS</div>
			<jqxSortable class='panel-body' [handle]='".handle"' (onStop)="orderChanged()">
				<div *ngFor="let room of _rooms">
					<room-editor [room]="room" (onDeleted)="onRoomDelete($event)"></room-editor>
				</div>
			</jqxSortable>
		</div>
		<button type="button" class="btn btn-default" (click)="onNewRoom()" translate>NEWROOM</button>
		<a role="button" class="btn btn-default" [routerLink]="['/cfg/roomassign']" translate>ASSIGNDEVS</a>
	`
})
export class RoomsComponent implements OnInit {
	private _rooms: Device[];
	private _headerIcon = "jqx-icon-close";
	private _canCollapse = false;

	constructor(private el: ElementRef, private _route: ActivatedRoute, private _router: Router,
		private _chsService: ChsService) {
	}


	private initData() {
		this._rooms = sortDevs(this._chsService.Rooms.values());
	}

	ngOnInit() {
		this._chsService.getDevices().subscribe(next => { }, err => { }, () => {
			this.initData();
		});
	}

	private onNewRoom() {
		this._chsService.newRoom().subscribe(dev => {
			this._rooms.push(dev);
		});
	}

	onRoomDelete(id: string) {
		this._chsService.deleteRoom(id).subscribe( () => {
			this._rooms = this._rooms.filter(s => s.id!==id );
		});
	}

	private orderChanged() {
		let items=jQuery(this.el.nativeElement).find(".panel.panel-default[id]");
		var ar:Device[]=[];

		for(var index = 0; index < items.length; index++) {
			var element = jQuery(items[index]);
			var dev = this._chsService.Rooms.get(element.attr("id"));
			dev.sort = index;
			ar.push(dev);
		}
			
		this._chsService.writeDevices(ar).subscribe(next => {}, err => {}, () => {});
	}


}