import { Component, ElementRef, Input, OnInit, ViewChild } from "@angular/core";
import { ChsService } from "../chs.service";
import { Device, Member } from "../devices/device";
import { Translation, TranslationService } from "angular-l10n";

@Component({
	moduleId: module.id,
	selector: "devtarget-editor",
	templateUrl: "devtargeteditor.component.html"
})
export class DevtargeteditorComponent   implements OnInit {
	@Input() public dev: Device;
	@Input() public scene: Device;

	private _targetState: string;
	private _targetPos: number;
	private _devType: string;
	private _dropDownTargetSource = [ { label: "", target: "driveUp" }, { label: "", target: "drivePos2" }, 
																		{ label: "", target: "drivePos1" }, { label: "", target: "driveDown" }, { label: "", target: "drivePos " } ];


	constructor(public translation: TranslationService, private el: ElementRef, private _chsService: ChsService) {
		 
	}

	ngOnInit() {
		let dtTargets: any = this._dropDownTargetSource;
		dtTargets[0].label=this.translation.translate("POSUP");
		dtTargets[1].label=this.translation.translate("POSAIR");
		dtTargets[2].label=this.translation.translate("POSINT");
		dtTargets[3].label=this.translation.translate("POSDOWN");
		dtTargets[4].label=this.translation.translate("POSIND");


		this._devType=ChsService.getTypeState(this.dev.type);
		let m = (<Member[]>this.scene.members).find((m, idx, obj) => (m.id === this.dev.id));
		if (this._devType==="slider") {
			this._targetPos = 50;
			this._targetState = "driveUp";
		}	else {
			this._targetState = "off";
		}
		if (m) { this._targetState = m.value.toString(); }
		if (this._targetState.startsWith("drivePos ")) {
			this._targetPos = Number.parseInt(this._targetState.substring(9));
			this._targetState = "drivePos ";
		}
	}

	private saveDev(): void {
		let members: Member[] = <Member[]>this.scene.members;
		let index: number = -1;
		let m = members.find((m, idx, obj) => {
			if (m.id === this.dev.id) { index = idx; return true; }
			return false;
		});

		if (m) {
			m.value = this._targetState;
			if (this._targetState === "drivePos ") {
				m.value = "drivePos " + this._targetPos.toString();
			}
		}
		this._chsService.writeDevice(this.scene).subscribe();
	}

	private onChangeTarget(evt: any): void {
		this._targetState = this._dropDownTargetSource[evt.index].target;
		this.dev.device_state = this._targetState;
		if (this._targetState === "drivePos ") {
			this.dev.device_state = "drivePos " + this._targetPos.toString();
		}
		this.saveDev();
	}

	private onChangeTargetSwitch(pos: string): void {
		this._targetState = pos;
		this.dev.device_state = this._targetState;
		this.saveDev();
	}	

	private onTargetPosChanged(args: any): void {
		let pos: number = Number.parseInt(args.value);
		if (Number.isNaN(pos)) { return; }
		if (pos !== this._targetPos && pos>=0 && pos<=100) {
			this._targetPos = pos;
			this.dev.device_state = "drivePos " + this._targetPos.toString();
			this.saveDev();
		}
	}

}