"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../utils");
var SceneDataAdapter = (function () {
    function SceneDataAdapter(_chsService) {
        this._chsService = _chsService;
        this.source = {
            localData: new Array(),
            dataType: "array",
            dataFields: [
                { name: "id", type: "string" },
                { name: "status", map: "scene", type: "string" },
                { name: "text", map: "name", type: "string" },
                { name: "tags", map: "type", type: "string" },
                { name: "resourceId", map: "type", type: "string" },
                { name: "origid", type: "string" },
                { name: "color", type: "string" },
                { name: "targetState", type: "string" }
            ]
        };
    }
    SceneDataAdapter.prototype.setDevices = function (filter) {
        var _this = this;
        this.source.localData = [];
        this._chsService.RealDevs.forEach(function (dev) {
            if (!filter || filter == dev.type)
                _this.source.localData.push({
                    id: dev.id.replace(".", "_"), origid: dev.id, name: dev.name, scene: dev.device,
                    type: dev.type, color: utils_1.backcolor
                });
        });
    };
    SceneDataAdapter.prototype.memberMap = function (scene) {
        var members = scene.members;
        var map = new Map();
        members.forEach(function (m) { return map.set(m.id, m.value); });
        return map;
    };
    SceneDataAdapter.prototype.setDevicesForScene = function (sceneid) {
        var _this = this;
        var sceneDev = this._chsService.getCachedDev(sceneid);
        var members = sceneDev.members;
        var mapMembers;
        if (members)
            mapMembers = this.memberMap(sceneDev);
        else
            mapMembers = new Map();
        this.source.localData = [];
        this._chsService.RealDevs.forEach(function (dev) {
            var s = "noscene";
            if (mapMembers.has(dev.id))
                s = sceneid;
            _this.source.localData.push({
                id: dev.id.replace(".", "_"), origid: dev.id, name: dev.name, scene: s, type: dev.type, color: utils_1.backcolor,
                targetState: mapMembers.get(dev.id)
            });
        });
    };
    SceneDataAdapter.prototype.dataAdapter = function () {
        var da = new $.jqx.dataAdapter(this.source);
        return da;
    };
    SceneDataAdapter.prototype.resources = function () {
        return utils_1.resources;
    };
    return SceneDataAdapter;
}());
exports.SceneDataAdapter = SceneDataAdapter;
//# sourceMappingURL=sceneDataAdapter.js.map