import { ChsService } from "../chs.service";
import { Device, Member } from "../devices/device";
import { resources, backcolor } from "../utils";

export class SceneDataAdapter {
		constructor(private _chsService: ChsService) { }

	private source =
	{
		localData: new Array<any>(),
		dataType: "array",
		dataFields: [
			{ name: "id", type: "string" },
			{ name: "status", map: "scene", type: "string" },
			{ name: "text", map: "name", type: "string" },
			{ name: "tags", map: "type", type: "string" },
			{ name: "resourceId", map: "type", type: "string" },
			{ name: "origid", type: "string" },
			{ name: "color", type: "string" },
			{ name: "targetState", type: "string" }
		]
	};

	public setDevices(filter?: string) {
		this.source.localData = [];
		this._chsService.RealDevs.forEach(dev => {
			if (!filter || filter == dev.type)
				this.source.localData.push({
					id: dev.id.replace(".", "_"), origid: dev.id, name: dev.name, scene: dev.device,
					type: dev.type, color: backcolor
				});
		});
	}

	private memberMap(scene: Device): Map<string, string> {
		let members = <Member[]>scene.members;
		let map = new Map<string, string>();
		members.forEach(m => map.set(m.id, <string>m.value));
		return map;
	}

	public setDevicesForScene(sceneid: string) {
		let sceneDev = this._chsService.getCachedDev(sceneid);
		let members = <Member[]>sceneDev.members;
		var mapMembers: Map<string, string>;
		if (members)
			mapMembers = this.memberMap(sceneDev);
		else
			mapMembers = new Map<string, string>();
		this.source.localData = [];
		this._chsService.RealDevs.forEach(dev => {
			let s = "noscene";
			if (mapMembers.has(dev.id))
				s = sceneid;
			this.source.localData.push({
				id: dev.id.replace(".", "_"), origid: dev.id, name: dev.name, scene: s, type: dev.type, color: backcolor,
				targetState: mapMembers.get(dev.id)
			});
		});
	}



	public dataAdapter(): any {		
		var da = new $.jqx.dataAdapter(this.source);
		return da;
	}

	public resources() {
		return resources;
	}
}