import { Component, ElementRef, EventEmitter, Input, Output, OnInit, ViewChild } from "@angular/core";
import { Router } from "@angular/router";
import { ChsService } from "../chs.service";
import { Device, Member } from "../devices/device";

// <a (click)="onGotoMembers()"><img src='css/images/icon-right-transparent.png'/></a>
//				<a [routerLink]="['/Gruppen', 'Gmembers', {id: _id}]" (click)='onGotoMembers()'><img src='css/images/icon-right-transparent.png'/></a>

@Component({
	selector: "scene-editor",
	template: `
			<div  id="{{dev.id}}" class='panel panel-default commeo-icon-container' style='margin-bottom: 8px;'><div class='panel-body'>
				<div class='commeo-icon-right1 jqx-icon jqx-icon-close' (click)='onClose()'></div>
				<div class='handle commeo-icon-left1 jqx-icon jqx-icon-sort-handle'></div>
				<p></p>
				<div class="input-group">
  				<input type="text" class="form-control" aria-label="..." [(value)]='dev.name' (focus)='onFocused($event)' (change)='onValueChanged()' #nameInput>
  				<div class="input-group-btn">
    				<button type="button" class="btn btn-default" (click)='onEditToggle()'><div class='glyphicon' [class.glyphicon-pencil]='!_edit' [class.glyphicon-ok]='_edit'></div></button>
    				<button type="button" class="btn btn-default" (click)='onFavChange()'><div class='glyphicon' [class.glyphicon-star-empty]='!dev.favorite' [class.glyphicon-star]='dev.favorite' ></div></button>
    				<button type="button" class="btn btn-default" (click)='onGotoMembers()'><div class='glyphicon glyphicon-menu-right'></div></button>
  				</div>
				</div>
				<p></p>
				<div class="btn-group" role="group" aria-label="...">
					<button type="button" class="btn btn-default" (click)='onStopScene()'><div class='glyphicon glyphicon-stop'></div></button>
					<button type="button" class="btn btn-default" (click)='onStartScene()'><div class='glyphicon glyphicon-play'></div></button>
				</div>
				<p></p>
				<jqxExpander [expanded]='false'>
					<div><span translate>SWITCHTIMES</span>					
						<button (click)='onTimers()' class='glyphicon glyphicon-pencil'></button>
					</div>
					<ul class="list-group">
						<li *ngFor="let timer of _timers" class='list-group-item'>
							<timer-display [timer]="timer"></timer-display>
						</li>
					</ul>
				</jqxExpander>
			</div></div>
	`
})
export class SceneeditorComponent implements OnInit {
	@Input()	public dev: Device;
	@Output() onDeleted = new EventEmitter<string>();
	
	@ViewChild("nameInput") private _input: ElementRef;
	private _edit: boolean=false;
	private _oldName: string;
	private _state: string;
	private _id: string;
	private _buttonChecked: boolean;
	private _sliderValue: number;
	private _timers: Device[] = [];
	
	
	constructor(private _chsService: ChsService, 
							private _router: Router) {
	}
	
	ngOnInit() {
		this._id=this.dev.id.replace(".", "__");
		this._timers=[];
		this._chsService.Timers.forEach(dev => {  if(dev.type=="timer" && dev.device==this.dev.id) this._timers.push(dev); } );
	}
	
	onClose() {
		this.onDeleted.emit(this.dev.id);
	}
	
	onEditToggle() {
		// wird nicht aufgerufen, wenn der onValueChange-Handler aktiv ist
		if (this._edit) {
			this._edit=false;
			if (this._oldName!=this.dev.name) {
				this._chsService.writeDevice(this.dev).subscribe(() => {});
			}
		}
		else {
			this._edit=true;
			this._oldName=this.dev.name;
			this._input.nativeElement.focus();
		}
	}

	onGotoMembers() {
		this._router.navigate(["cfg/scenemembers", this._id]);
	}

	onTimers() {
		this._router.navigate(["cfg/timers", this._id]);
	}

	onFavChange() {
		this.dev.favorite=!this.dev.favorite;
		this._chsService.setFavorite(this.dev.id, this.dev.favorite);
	}

	private drive(cmd: string) {
		this._chsService.setDevState(this.dev.id, cmd).subscribe(() => { });
	}
	

	onStartScene() {
		this.drive("on");
		/*
		if (!this.dev.members || this.dev.members.length==0)
			return;
			
		var obs: Observable<any>=null;
		(<Member[]>this.dev.members).forEach( m => {
			if (obs)
				obs=obs.merge(this._chsService.setDevState(m.id, m.value.toString()));
			else
				obs=this._chsService.setDevState(m.id, m.value.toString());
		});
		
		if (obs) obs.subscribe();		*/
	}

	onStopScene() {
		this.drive("stop");

		/*
		if (!this.dev.members || this.dev.members.length==0)
			return;
			
		var obs: Observable<any>=null;
		(<Member[]>this.dev.members).forEach( m => {
			if (obs)
				obs=obs.merge(this._chsService.setDevState(m.id, 'stop'));
			else
				obs=this._chsService.setDevState(m.id, 'stop');
		});
		
		if (obs) obs.subscribe();		*/
	}


	onFocused(event: any) {
		if (!this._edit) {
			this._oldName=this.dev.name;
			this._edit=true;
		}
	}

	onValueChanged() {
		if (this._input.nativeElement.value!=this.dev.name)
			this.dev.name=this._input.nativeElement.value;
		if (this._edit)
			this.onEditToggle();
	}


}