import { Component, OnInit, ViewChild } from "@angular/core";
import { Router, ActivatedRoute } from "@angular/router";
import { Translation, LocaleService, TranslationService } from "angular-l10n";
import { ChsService } from "../chs.service";
import { TranslateService } from "../translation.service";
import { SceneDataAdapter } from "./sceneDataAdapter";
import { Device } from "../devices/device";
import { jqxKanbanComponent } from "../jqwidgets-ts/angular_jqxkanban";

@Component({
	template: `
				<div class='panel panel-primary'>
					<div class='panel-heading'><span translate>ASSIGNMEMBERS</span> {{_scene?.name}}</div>
					<div class='panel-body'>
						<div *ngIf='_scene'>
							<jqxKanban
								[width]="'100%'" [itemRenderer]='_itemRenderer'
								[source]='_source' [template]='_template' [resources]='_resources'
								[columns]='_columns' (onItemMoved)='onItemMoved($event)'>
							</jqxKanban>
						</div>
					</div>
				</div>
				<a role="button" class="btn btn-default" [routerLink]="['/cfg/scenes']" translate>BACK</a>
				<button type="button" class="btn btn-default" (click)="onTargets()" translate>CONFIGTARGETS</button>
      `
})
export class ScenemembersComponent   implements OnInit {
	@ViewChild(jqxKanbanComponent) _kanban: jqxKanbanComponent;
	private _id: string;
	private _scene: Device;
	private _da: SceneDataAdapter;
	private _columns = [{ text: "DEVICES", dataField: "noscene" }, { text: "", dataField: "" }];
	private _source: any;
	private _resources: any;
	private _template = `
								<div groups class='jqx-kanban-item' id=''>
									<div groups class='jqx-kanban-item-color-status'></div>
									<div groups style='display: none' class='jqx-kanban-item-avatar'></div>
									<div groups class='jqx-kanban-item-text'></div>
									<div groups style='display: none' class='jqx-kanban-item-footer'></div>
								</div>
							`;
	private _itemRenderer = function (element: any, item: any, resource: any) {
		let bInScene=(item.status!=="noscene");
		jQuery(element).find(".jqx-kanban-item-color-status").html(
			"<img class='menu-icon' src='"+resource.image +"' height='16px' /><span style='line-height: 23px; margin-left: 5px; color:white;'>" + resource.name + "</span>");
	 	var textElem=jQuery(element).find(".jqx-kanban-item-text");
	  if (!bInScene) {
			textElem.html(item.text);
		} else if (item.targetState) {
			let strHtml="";
			if (item.targetState==="driveUp") strHtml="Obere Endlage";
			else if (item.targetState==="driveDown") strHtml="Untere Endlage";
			else if (item.targetState==="drivePos1") strHtml="Zwischenposition";
			else if (item.targetState==="drivePos2") strHtml="Lüftungsposition";
			else if ((<string>(item.targetState)).startsWith("drivePos")) strHtml=(<string>(item.targetState)).substring(8)+"%";
			
			textElem.html(item.text+ "<p>" + strHtml +"</p>");
		}
	};

	constructor(public translation: TranslationService,
							private _route: ActivatedRoute, private _router: Router,
							private _chsService: ChsService, private _transService: TranslateService) {
		 
		this._id = this._route.snapshot.params["id"].replace("__", ".");
		this._da = new SceneDataAdapter(_chsService);
		this._resources = this._da.resources();
	}

	ngOnInit() {
		let s = this._chsService.getCachedDev(this._id);
		if (s) {
			this.initData(s);
		}
		else {
			this._chsService.getDevices().subscribe(x => {},
				error => {},
				() => {
					s=this._chsService.getCachedDev(this._id);
					this.initData(s);
				}
			);
		}
	}

	private initData(s: Device) {
		this._scene = s;
		this._da.setDevicesForScene(this._scene.id);
		this._source = this._da.dataAdapter();
		this._columns[0].text = this.translation.translate("DEVS");
		this._columns[1].text = this._scene.name;
		this._columns[1].dataField = this._scene.id;
	}

	onItemMoved(event: any) {
		var args = event.args;
		if (args.oldColumn.dataField !== args.newColumn.dataField) {
			let itemData = args.itemData;
			// Originale Id holen aus DataAdapter
			let records: any[] = this._source.records;
			var origid: string = "";
			records.forEach(item => {
				if (item.id === itemData.id) {
					origid = item.origid;
					this._kanban.updateItem(item.id, item);
				}
			});

			if (args.newColumn.dataField === "noscene")
				this._chsService.removeFromScene(origid, args.oldColumn.dataField).subscribe(() => { });
			else
				this._chsService.addToScene(origid, args.newColumn.dataField, "driveUp").subscribe(() => { });
		}
	}

	onTargets() {
		this._router.navigate(["cfg/scenetargets", this._id.replace(".", "__")]);
	}
}