import { Component, AfterViewInit, ElementRef } from "@angular/core";
import { ChsService } from "../chs.service";
import { Device } from "../devices/device";
import { SceneDataAdapter } from "./sceneDataAdapter";

@Component({
	template: `
			<div class='panel panel-primary'>
				<div class='panel-heading' translate>SCENES</div>
				<jqxSortable id='scenelist' class='panel-body' [handle]='".handle"' (onStop)="orderChanged()">
					<div *ngFor="let scene of _scenes">
						<scene-editor [dev]="scene" (onDeleted)="onSceneDelete($event)"></scene-editor>
					</div>
				</jqxSortable>
			</div>
			<button type="button" class="btn btn-default" (click)="onNewScene()" translate>NEWSCENE</button>
			`
})
export class ScenesComponent implements AfterViewInit {
	constructor(private el: ElementRef, private _chsService: ChsService) {
		this._da = new SceneDataAdapter(_chsService);
	}

	public _da: SceneDataAdapter;

	private _scenes: Device[] = [];

	ngAfterViewInit() {
		this._chsService.getDevices().subscribe(devs => {
			this._da.setDevices();
			try {
				this._scenes=[];
				this._chsService.Scenes.forEach(dev => {  if (dev.type=="scene") this._scenes.push(dev); } );
			} catch (ex) {}
		},
		error => {});
	}

	onNewScene() {
		this._chsService.newScene().subscribe( scene => {
			this._scenes.push(scene);
		});
	}
	
	onSceneDelete(id: string) {
		this._chsService.deleteScene(id).subscribe( () => {
			this._scenes = this._scenes.filter(s => s.id!=id );
		});
	}

	private orderChanged() {
		let items=jQuery(this.el.nativeElement).find(".panel.panel-default[id]");
		var ar:Device[]=[];

		for(var index = 0; index < items.length; index++) {
			var element = jQuery(items[index]);
			var dev = this._chsService.Scenes.get(element.attr("id"));
			dev.sort = index;
			ar.push(dev);
		}

		this._chsService.writeDevices(ar).subscribe(next => {}, err => {}, () => {});
	}

}