import { Component, OnInit, AfterViewInit } from "@angular/core";
import { FormBuilder } from "@angular/forms";
import { ChsService } from "../chs.service";
import { SettingsService } from "./settings.service";
import { SettingTime } from "./setting.models";
import * as moment from "moment";

@Component({
	moduleId: module.id,
	selector: "clock-setting",
	templateUrl: "setting.clock.component.html"
})
export class SettingClockComponent implements AfterViewInit {
	private _dtTime: any;
	private _model: SettingTime = { ntp: false, time: 0, timezone: "Europe/Berlin", zonelist: ["Europe/Berlin", "Europe/Amsterdam"] };
	private _tzIndex: number;

	constructor(private _chsService: ChsService, private _settingsService: SettingsService, private formBuilder: FormBuilder) {
	}

	ngAfterViewInit() {
		this._chsService.getDateTime().subscribe(timeSettings => {
			if (timeSettings) {
				this._model = timeSettings;
				let m = moment.unix(timeSettings.time);
				this._dtTime= m.toDate();
				if (this._model.ntp===undefined)
					this._model.ntp=false;
				if (this._model.timezone===undefined)
					this._model.timezone="";

				if (!this._model.zonelist) {
					this._model.zonelist=["Europe/Berlin", "Europe/Amsterdam"];
				}

				for (let i:number = 0; i<this._model.zonelist.length; ++i) {
					if (this._model.timezone===this._model.zonelist[i]) {
						this._tzIndex=i;
						break;
					}
				}
			}
		},
		err => {}, () => {});
	}

	private onDateTimeChanged(event: any) {
		this.onDtChange(event.args);
	}

	private onTzChanged(event: any) {
		this._tzIndex=event.args.index;
		this._model.timezone=this._model.zonelist[this._tzIndex];
	}

	private onDtChange(args: any) {
		if (args.oldValue instanceof Date && !moment(args.date).isSame(args.oldValue)) {
			this._dtTime=args.date;
		}
	}

	// tslint:disable-next-line:no-unused-variable
	private onSubmit(): void {
		/*let st: SettingTime = { ntp: this._model.ntp, time: moment(this._dtTime).unix(),
			timezone: this._model.timezone };*/
		let st: SettingTime = { ntp: this._model.ntp, time: moment(this._dtTime).unix(),
			timezone: this._model.timezone };
		this._chsService.setDateTime(st).subscribe();
	}

}