import { Component } from "@angular/core";
import { FormBuilder } from "@angular/forms";
import { Translation, LocaleService, TranslationService } from "angular-l10n";
import { ChsService } from "../chs.service";
import { SettingsService } from "./settings.service";
import { ILanguage } from "./setting.models";

@Component({
	moduleId: module.id,
	selector: "language-setting",
	templateUrl: "setting.language.component.html"
})
export class SettingLanguageComponent   {
	private _language: ILanguage;
	private _dropDownLanguageSource: any[];
	private _index: number;
	private _browserLang: ILanguage;



	constructor(public locale: LocaleService, public translation: TranslationService,
		private _chsService: ChsService, private _settingsService: SettingsService, private formBuilder: FormBuilder) {
		 
		this._browserLang = this._settingsService.getBrowserLanguage();
		this._dropDownLanguageSource = this._settingsService.LANGUAGES;
		let lang: string = this.locale.getCurrentLanguage();
		if (lang) {
			this._index = this._settingsService.getLanguageIndex(lang);
		}
		this._language = this._settingsService.LANGUAGES[this._index];

		this.locale.languageCodeChanged.subscribe(() => {
			location.reload(true);
		});
	}

	private onSubmit(): void {
		this.locale.setCurrentLanguage(this._language.iso);
	}

	private onLanguageChanged(index: number): void {
		this._index = index;
		this._language = this._settingsService.LANGUAGES[this._index];
	}
}