import { Component, AfterViewInit, ViewChild } from "@angular/core";
import { FormBuilder } from "@angular/forms";
import { Translation, TranslationService } from "angular-l10n";
import { ChsService } from "../chs.service";
import { SettingsService } from "./settings.service";
import { Location } from "./setting.models";
import { jqxDropDownListComponent } from "../jqwidgets-ts/angular_jqxdropdownlist";
import { jqxNumberInputComponent } from "../jqwidgets-ts/angular_jqxnumberinput";

@Component({
	moduleId: module.id,
	selector: "location-setting",
	templateUrl: "setting.location.component.html"
})
export class SettingLocationComponent   implements AfterViewInit {
	private static _SettingId = "location";
	private _index = 0;
	private _location: Location = this._settingsService.CITIES[1];
	private _dropDownCitySource = this._settingsService.CITIES;
	@ViewChild("dropDownLoc") private _dropDownLoc: jqxDropDownListComponent;
	@ViewChild("numberInputLat") private _numberInputLat: jqxNumberInputComponent;
	@ViewChild("numberInputLong") private _numberInputLong: jqxNumberInputComponent;

	constructor(public translation: TranslationService,
		private _chsService: ChsService, private _settingsService: SettingsService, private formBuilder: FormBuilder) {
		 
		this._location = <Location>this._chsService.getStoredSetting(SettingLocationComponent._SettingId);
		this.calcIndex();
	}

	private calcIndex(): void {
		if (this._location && this._location.city) {
			for (var i: number = 0; i < this._settingsService.CITIES.length; ++i) {
				if (this._settingsService.CITIES[i].key === this._location.key) {
					this._index = i;
					i = this._settingsService.CITIES.length;
				}
			}
		} else {
			this._index = 1; // Kassel ist default
			if (!this._location) {
				this._location = this._settingsService.CITIES[this._index];
			}
		}

	}
	public ngAfterViewInit() {
		this._index = 0;
		this._location = this._settingsService.CITIES[this._index];
		this._dropDownLoc.createComponent();
		//this._numberInputLat.createComponent();
		//this._numberInputLong.createComponent();
		this._chsService.onGotSettings.subscribe(() => {
			this._location = <Location>this._chsService.getStoredSetting(SettingLocationComponent._SettingId);
			this.calcIndex();
			this._dropDownLoc.selectedIndex(this._index);
			//console.debug("Location " + this._index + ": " + this._location.city);
		});
	}

	private onSubmit(): void {
		if (this._index==0) {
			this._location.latitude=this._numberInputLat.getDecimal();
			this._location.longitude=this._numberInputLong.getDecimal();
		}

		this._chsService.setSetting(SettingLocationComponent._SettingId, this._location).subscribe();
	}

	private onCityChanged(event: number): void {
		this._index = event;
		if (this._index !== 0) {
			this._location = this._settingsService.CITIES[this._index];
		} else {
			this._location.city = this._settingsService.CITIES[0].city;
			this._location.key = this._settingsService.CITIES[0].key;
		}
		console.debug("Location " + this._index + ": " + this._location.city);
	}

}