import { Injectable } from "@angular/core";
import { Translation, TranslationService } from "angular-l10n";
import { Location, ILanguage } from "./setting.models";

@Injectable()
export class SettingsService   {

	public CITIES: Location[] = [
		{ key: "CUSTOMLOC", city: "Eigene Angaben", latitude: 0, longitude: 0 },
		{ key: "KASSEL", city: "Kassel", latitude: 51.3, longitude: 9.5 },
		{ key: "BERLIN", city: "Berlin", latitude: 52.2, longitude: 13.4 },
		{ key: "HAMBURG", city: "Hamburg", latitude: 53.6, longitude: 10.0 },
		{ key: "COLOGNE", city: "Köln", latitude: 50.9, longitude: 7 },
		{ key: "STUTTGART", city: "Stuttgart", latitude: 48.8, longitude: 9.2 },
		{ key: "MUNICH", city: "München", latitude: 48.1, longitude: 11.6 },
		{ key: "VIENNA", city: "Wien", latitude: 48.2, longitude: 16.4 },
		{ key: "KLAGENFURT", city: "Klagenfurt", latitude: 46.6, longitude: 14.3 },
		{ key: "BERN", city: "Bern", latitude: 46.9, longitude: 7.4 },
		{ key: "PARIS", city: "Paris", latitude: 48.8, longitude: 2.4 },
		{ key: "RENNES", city: "Rennes", latitude: 48.1, longitude: -1.7 },
		{ key: "BORDEAUX", city: "Bordeaux", latitude: 44.8, longitude: -0.6 },
		{ key: "LYON", city: "Lyon", latitude: 45.8, longitude: 4.8 },
		{ key: "BRUSSELS", city: "Brüssel", latitude: 50.8, longitude: 4.4 },
		{ key: "AMSTERDAM", city: "Amsterdam", latitude: 52.4, longitude: 4.9 },
		{ key: "WARSAW", city: "Warschau", latitude: 52.2, longitude: 21 },
		{ key: "POZNAN", city: "Poznan", latitude: 52.4, longitude: 17 },
		{ key: "PRAGUE", city: "Prag", latitude: 50.1, longitude: 14.4 },
		{ key: "BUDAPEST", city: "Budapest", latitude: 47.5, longitude: 19 },
		{ key: "ZAGREB", city: "Zagreb", latitude: 45.8, longitude: 16 },
		{ key: "MADRID", city: "Madrid", latitude: 40.4, longitude: -3.7 },
		{ key: "BARCELONA", city: "Barcelona", latitude: 41.4, longitude: 2.2 },
		{ key: "LISBON", city: "Lissabon", latitude: 38.7, longitude: -9.1 }
	];

	public LANGUAGES: ILanguage[] = [
		//{ key: "LANGAUTO", language: "", iso: "", country: ""},
		{ key: "GERMAN", language: "", iso: "de", country: "DE"},
		{ key: "ENGLISH", language: "", iso: "en", country: "GB"},
		{ key: "FRENCH", language: "", iso: "fr", country: "FR"},
		{ key: "DUTCH", language: "", iso: "nl", country: "NL"},
		{ key: "POLISH", language: "", iso: "pl", country: "PL"},
		{ key: "CZECH", language: "", iso: "cs", country: "CZ"}
	];

	constructor(public translation: TranslationService) {
		 
		this.translateCities();
		this.translateLanguages();
		this.translation.translationChanged.subscribe(() => { 
			this.translateCities();
			this.translateLanguages();
		});
	}

	private translateCities(): void {
		this.CITIES.forEach(loc => {
			loc.city = this.translation.translate(loc.key);
		});
	}

	private translateLanguages(): void {
		this.LANGUAGES.forEach(lang => {
			lang.language = this.translation.translate(lang.key);
		});
	}

	public getBrowserLanguage(): ILanguage {
		let lang: string = navigator.language;
		let nMinus: number=lang.indexOf("-");
		if(nMinus>0) {
			lang=lang.substring(0, nMinus);
		}

		for (let i=0; i< this.LANGUAGES.length; ++i) {
			if (this.LANGUAGES[i].iso===lang) {
				return this.LANGUAGES[i];
			}
		}

		return this.LANGUAGES[2];
	}

	public getLanguage(lang: string): ILanguage {
		for (let i=0; i< this.LANGUAGES.length; ++i) {
			if (this.LANGUAGES[i].iso===lang) {
				return this.LANGUAGES[i];
			}
		}

		return null;
	}
	
		public getLanguageIndex(lang: string): number {
		for (let i=0; i< this.LANGUAGES.length; ++i) {
			if (this.LANGUAGES[i].iso===lang) {
				return i;
			}
		}

		return 0;
	}
}