import { Component, ElementRef, Input, OnInit } from "@angular/core";
import { TranslationService } from "angular-l10n";
import { Device } from "../devices/device";
import { ChsService } from "../chs.service";
import * as moment from "moment";

@Component({
	moduleId: module.id,
	selector: "timer-display",
	template: `
		<div>
			<jqxSwitchButton style='float: right; margin-right: -5px; margin-top: -5px;'
				[height]='24' [onLabel]='translation.translate("ON")' [offLabel]='translation.translate("OFF")' (onChange)="onSwitchChange($event)"
				[checked]="timer.state=='on'"></jqxSwitchButton>
		
			<div>{{ _timerTypeDescr }}</div>
			<div>{{ _timerDaysDescr }}</div>
			<div *ngIf="devType=='onoff' || devType=='slider'">
				{{ _timerTarget }}
			</div>
			<div>{{ _timerDetailsDescr }}</div>
		</div>
	`
})
export class TimerdisplayComponent implements OnInit {
	@Input() public timer: Device;
	@Input() public devType: string;
	private _timerTypeDescr: string;
	private _timerDaysDescr: string;
	private _timerDetailsDescr: string;
	private _timerTarget: string;

	public static _type = {
		"static": "TIMERTINDIVIDUAL", "random": "TIMERTRANDOM",
		"astroSunrise": "TIMERTASTROM", "astroSunset": "TIMERTASTROE"
	};
	private static _days = ["SUNABBR", "MONABBR", "TUEABBR", "WEDABBR", "THUABBR", "FRIABBR", "SATABBR"];

	constructor(public translation: TranslationService, private el: ElementRef, private _chsService: ChsService) {
		 
	}

	ngOnInit() {
		this._timerDaysDescr = this.daysToString(this.timer.days);
		this._timerTypeDescr = this.timerTypeDescr();
		this._timerDetailsDescr = this.timerDetailsDescr();
		this._timerTarget = this.timerTarget();
	}

	private TimeToString(tm: string): string {
		let fmt: string = "HH:mm";
		var ret: Date = new Date(1970, 1, 1, 12, 0, 0);
		if (!tm) { return moment(ret).format(fmt); }
		let splitted = tm.split(":");
		if (splitted.length !== 2) { return moment(ret).format(fmt); }

		return moment(new Date(1970, 1, 1, parseInt(splitted[0]), parseInt(splitted[1]), 0)).format(fmt);
	}

	private t(str: string): string {
		return this.translation.translate(str);
	}

	private timerTypeDescr(): string {
		if (!this.timer) { return "null"; }
		let ret: string = this.t(TimerdisplayComponent._type[this.timer.timer_type]);
		switch (this.timer.timer_type) {
			case "static": ret += ": " + this.TimeToString(this.timer.time); break;
			case "random": ret += ": " + this.TimeToString(this.timer.time); break;
			case "astroSunrise":
				if (this.timer.time) {
					ret += " " + this.t("NOTBEFORE") + " " + this.TimeToString(this.timer.time);
					if (this.timer.offset) {
						ret += ", " + this.t("ASTROOFFSET") + " " + this.timer.offset.toString() + " " + this.t("MINUTES");
					}
				}
				break;
			case "astroSunset":
				if (this.timer.time) {
					ret += " " + this.t("NOTAFTER") + " " + this.TimeToString(this.timer.time);
					if (this.timer.offset) {
						ret += ", " + this.t("ASTROOFFSET") + " " + this.timer.offset.toString() + " " + this.t("MINUTES");
					}
				}
				break;
		}

		return ret;
	}

	private timerDetailsDescr(): string {
		if (!this.timer) { return "null"; }
		var ret: string = "";
		if (this.timer.readings) {
			let next = this.timer.readings["next"];
			if (next) {
				ret += this.t("NEXTSWITCHTIME") + " " + moment(next).format("HH:mm DD.MM.YYYY");
			}
		}

		return ret;
	}

	private timerTarget(): string {
		if (!this.timer) { return "null"; }
		var ret: string = "";

		if (this.devType === "slider" || this.devType === "onoff") {
			switch (this.timer.device_state) {
				case "on": ret += this.t("SWITCHON"); break;
				case "off": ret += this.t("SWITCHOFF"); break;
				case "driveUp": ret += this.t("DRIVETO") + " " + this.t("POSUP"); break;
				case "driveDown": ret += this.t("DRIVETO") + " " + this.t("POSDOWN"); break;
				case "drivePos1": ret += this.t("DRIVETO") + " " + this.t("POSINT"); break;
				case "drivePos2": ret += this.t("DRIVETO") + " " + this.t("POSAIR"); break;
				default:
					if (this.timer.device_state.startsWith("drivePos ")) {
						ret += this.t("DRIVETO") + " " + this.timer.device_state.substring(9) + "%";
					}
			}
		}

		return ret;
	}

	public daysToString(days: string): string {
		if (!days || days.length !== 7) { return; }
		let retVal: string = "";
		let i: number = 0;
		while (i < 7) {
			if (days[i] === "1") {
				retVal += this.t(TimerdisplayComponent._days[i]) + ", ";
			}
			++i;
		}
		if (retVal.length > 2) {
			retVal = retVal.substr(0, retVal.length - 2);
		}
		return retVal;
	}

	private saveTimer(): void {
		this._chsService.writeDevice(this.timer).subscribe();
	}


	onSwitchChange(val: any): void {
		this.timer.state = (val.args.checked ? "on" : "off");
		this.saveTimer();
	}

}