import { Component, OnInit } from "@angular/core";
import { Router, ActivatedRoute } from "@angular/router";
import {ChsService} from "../chs.service";
import {Device} from "../devices/device";


@Component({
	template: `
			<div class='panel panel-primary'>
				<div class='panel-heading'><span translate>SWITCHTIMESFOR</span> {{_device?.name}}</div>
				<div class='panel-body'>
					<div *ngFor="let timer of _timers" class='panel panel-default'>
						<timer-editor [timer]="timer" [devType]="_devType" (onDeleted)="onTimerDelete($event)"></timer-editor>
					</div>
				</div>
			</div>
			<br />
			<button type="button" class="btn btn-default btn-sm" (click)="onNewTimer()" translate>NEWSWITCHTIME</button>
			<button type="button" class="btn btn-default btn-sm" (click)="onBack()" translate>BACK</button>
			`
})
export class TimersComponent implements OnInit {
	constructor(private _chsService: ChsService, private _route: ActivatedRoute, private _router: Router) {
		this._deviceid = this._route.snapshot.params["id"].replace("__", ".");
	}

	private _deviceid: string;
	private _devType: string;
	private _device: Device;
	private _timers: Device[] = [];
	
	ngOnInit() {
		let comp=this;
		this._chsService.getDevices().subscribe(devs => {
			try {
				comp._device=comp._chsService.getCachedDev(comp._deviceid);
				comp._timers=[];
				comp._chsService.Timers.forEach(dev => {  if(dev.type=="timer" && dev.device==comp._deviceid) comp._timers.push(dev); } );
				if (this._device.type=="group")
					comp._devType=this._chsService.getDeviceState(this._device.id);
			} catch (ex) {
			}
		},
		error => {});
	}



	onNewTimer() {
		this._chsService.newTimer(this._deviceid).subscribe( timer => {
			this._timers.push(timer);
		});
	}
	 
	onTimerDelete(id: string) {
		this._chsService.deleteTimer(id).subscribe( () => {
			this._timers = this._timers.filter(s => s.id!=id );
		});
	}
	
	onBack() {
		if (this._device.type=="scene")
			this._router.navigate(["/cfg/scenes"]);
		else
			this._router.navigate(["/cfg/groups"]);
	}


}