import { Component, OnInit, OnDestroy } from "@angular/core";
import { Response } from "@angular/http";
import { Router } from "@angular/router";
import { ChsService } from "../chs.service";
import { Version } from "../version";
import {Subscription} from "rxjs/Subscription";
import "bootstrap";

@Component({
	moduleId: module.id,
	templateUrl: "update.component.html"
})
export class UpdateComponent implements OnInit, OnDestroy {
	public version: Version = { version: "", server: "", update: false };
	private _updating: boolean = false;
	private _subPoll: Subscription = null;

	constructor(private _chsService: ChsService,  private _router: Router) {
	}

	getVersion(): void {
		this._chsService.getVersion().subscribe(
			ver => this.version = ver			);
	}

	ngOnInit(): void  {
		if (!this._chsService.GotVersionInfo) {
			this.getVersion();
		}	else {
			this.version=this._chsService.VersionInfo;
		}

		this._chsService.onError.subscribe((res: Response) => {	this.onChsServiceError(res); });
	}

	ngOnDestroy(): void {
		if (this._subPoll) {
			this._subPoll.unsubscribe();
		}
	}

	onChsServiceError(res: Response): void {
		if (res.status===0) {
			this._chsService.showMessageBox("Server nicht erreichbar");
		} else {
			this._chsService.showMessageBox("Server-Fehler: ("+ res.status + ") "+res.statusText);
		}
	}

	onUpdate():void {
		this._chsService.startUpdate().subscribe();
		this._updating=true;
		this._chsService.pollServerVersion( () => { window.location.href = "/"; } );
	}
}


