import {Device} from "./devices/device";
import { jqxKanbanComponent } from "./jqwidgets-ts/angular_jqxkanban";

export const global_theme: string = "web";
export const backcolor: string = "#69AC13";
//export const language: string = "language";


export function getSettings(elem: JQuery): any {
	let attrSettings = elem.attr("jqx-settings");

	let objSettings: any = {};
	if (attrSettings)
		objSettings = JSON.parse(attrSettings);

	if (!objSettings.theme)
		objSettings.theme = global_theme;

	return objSettings;
}

export function sortDevs(devs: IterableIterator<Device>): Device[] {
	let ar=Array<Device>();
	let i=devs.next();
	while (!i.done) {
		ar.push(i.value);
		i=devs.next();
	}

	ar = ar.sort((a, b) =>  {
		if (a.sort!=undefined && b.sort!=undefined)
			return a.sort-b.sort;
		return a.name.localeCompare(b.name);
	});
	return ar;
}

export function sortRealDevs(devs: IterableIterator<Device>): Device[] {
	let ar=Array<Device>();
	let i=devs.next();
	while (!i.done) {
		ar.push(i.value);
		i=devs.next();
	}

	ar = ar.sort((a, b) =>  {
		if (a.sort_dev!=undefined && b.sort_dev!=undefined)
			return a.sort_dev-b.sort_dev;
		return a.name.localeCompare(b.name);
	});
	return ar;
}

export function maxSort(map: Map<string, Device>): number {
	let maxSort = -1;

	map.forEach(dev => {
		if (dev.sort!=undefined && dev.sort > maxSort)
			maxSort=dev.sort;
	});

	return maxSort;
}

export function maxSortRealDev(map: Map<string, Device>): number {
	let maxSort = -1;

	map.forEach(dev => {
		if (dev.sort_dev!=undefined && dev.sort_dev > maxSort)
			maxSort=dev.sort_dev;
	});

	return maxSort;
}

export function strSetChar(str:string, index: number, c: string ): string {
	let retVal = '';
	if (index >= str.length)
		return str + c;

	if (index > 0)
		retVal = str.substring(0, index);
	retVal += c;

	retVal+= str.substring(index+1);
	return retVal; 
}

export function getSortedColumnItems(kanban: jqxKanbanComponent, col: string): any[] {
		let items: any[] = kanban.getColumnItems(col);
		let retItems: any[] = [];

		let itemsInOrder: JQuery = jQuery('[data-kanban-column-container="' + col + '"] .jqx-kanban-item .jqx-kanban-item-text', kanban.host);
		for (let i: number = 0; i < itemsInOrder.length; ++i) {
			let strItem: string = jQuery(itemsInOrder[i]).text();
			for (let k: number = 0; k < items.length; ++k) {
				if (<string>(items[k].text) === strItem) {
					retItems.push(items[k]);
					// console.debug(items[k]);
				}
			}
		}

		return retItems;
	}


export interface Resource {
	id: string;
	name: string;
	image: string;
}

export var resources: Resource[] = [
		{ id: "light", name: "Licht", image: "../../images/bulb.png" },
		{ id: "switch", name: "Schalter", image: "../../images/switch.png" },
		{ id: "night-switch", name: "Nachtschalter", image: "../../images/switch.png" },
		{ id: "dimmer", name: "Dimmer", image: "../../images/dimmer.png" },
		{ id: "shutter", name: "Rolllade", image: "../../images/shutter.png" },
		{ id: "marquee", name: "Markise", image: "../../images/marquee.png" },
		{ id: "jalousie", name: "Jalousie", image: "../../images/jalousie.png" },
		{ id: "scene", name: "Szene", image: "../../images/scene.png" },
		{ id: "group", name: "Gruppe", image: "../../images/group.png" }
];

